@@ -6,6 +6,7 @@
 -- License: MIT                                          --
 -- ----------------------------------------------------- --
 
+DROP TABLE IF EXISTS g_api_key;
 DROP TABLE IF EXISTS g_client_user_scope;
 DROP TABLE IF EXISTS g_scope_group_auth_scheme_module_instance;
 DROP TABLE IF EXISTS g_scope_group;
@@ -23,7 +24,9 @@ DROP TABLE IF EXISTS g_client;
 DROP TABLE IF EXISTS g_user_property;
 DROP TABLE IF EXISTS g_user_scope_user;
 DROP TABLE IF EXISTS g_user_scope;
+DROP TABLE IF EXISTS g_user_password;
 DROP TABLE IF EXISTS g_user;
+DROP TABLE IF EXISTS gpo_dpop;
 DROP TABLE IF EXISTS gpg_device_authorization_scope;
 DROP TABLE IF EXISTS gpg_device_authorization;
 DROP TABLE IF EXISTS gpg_access_token_scope;
@@ -32,6 +35,9 @@ DROP TABLE IF EXISTS gpg_refresh_token_scope;
 DROP TABLE IF EXISTS gpg_refresh_token;
 DROP TABLE IF EXISTS gpg_code_scope;
 DROP TABLE IF EXISTS gpg_code;
+DROP TABLE IF EXISTS gpo_par_scope;
+DROP TABLE IF EXISTS gpo_par;
+DROP TABLE IF EXISTS gpo_rar;
 DROP TABLE IF EXISTS gpo_device_scheme;
 DROP TABLE IF EXISTS gpo_device_authorization_scope;
 DROP TABLE IF EXISTS gpo_device_authorization;
@@ -52,7 +58,9 @@ DROP TABLE IF EXISTS gs_webauthn_credential;
 DROP TABLE IF EXISTS gs_webauthn_user;
 DROP TABLE IF EXISTS gs_otp;
 DROP TABLE IF EXISTS gs_user_certificate;
-DROP TABLE IF EXISTS gs_user_pkcs12;
+DROP TABLE IF EXISTS gpr_reset_credentials_email;
+DROP TABLE IF EXISTS gpr_reset_credentials_session;
+DROP TABLE IF EXISTS gpr_update_email;
 DROP TABLE IF EXISTS gpr_session;
 DROP TABLE IF EXISTS gs_oauth2_session;
 DROP TABLE IF EXISTS gs_oauth2_registration;
@@ -65,6 +73,7 @@ CREATE TABLE g_user_module_instance (
   gumi_display_name TEXT DEFAULT '',
   gumi_parameters TEXT,
   gumi_readonly INTEGER DEFAULT 0,
+  gumi_multiple_passwords INTEGER DEFAULT 0,
   gumi_enabled INTEGER DEFAULT 1
 );
 
@@ -143,6 +152,7 @@ CREATE TABLE g_scope_group (
   gsg_id INTEGER PRIMARY KEY AUTOINCREMENT,
   gs_id INTEGER,
   gsg_name TEXT NOT NULL,
+  gsg_scheme_required INTEGER DEFAULT 1,
   FOREIGN KEY(gs_id) REFERENCES g_scope(gs_id) ON DELETE CASCADE
 );
 
@@ -166,6 +176,18 @@ CREATE TABLE g_client_user_scope (
 CREATE INDEX i_g_client_user_scope_username ON g_client_user_scope(gcus_username);
 CREATE INDEX i_g_client_user_scope_client_id ON g_client_user_scope(gcus_client_id);
 
+CREATE TABLE g_api_key (
+  gak_id INTEGER PRIMARY KEY AUTOINCREMENT,
+  gak_token_hash TEXT NOT NULL,
+  gak_counter INTEGER DEFAULT 0,
+  gak_username TEXT NOT NULL,
+  gak_issued_at TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
+  gak_issued_for TEXT, -- IP address or hostname
+  gak_user_agent TEXT,
+  gak_enabled INTEGER DEFAULT 1
+);
+CREATE INDEX i_gak_token_hash ON g_api_key(gak_token_hash);
+
 CREATE TABLE g_client (
   gc_id INTEGER PRIMARY KEY AUTOINCREMENT,
   gc_client_id TEXT NOT NULL UNIQUE,
@@ -203,7 +225,6 @@ CREATE TABLE g_user (
   gu_username TEXT NOT NULL UNIQUE,
   gu_name TEXT DEFAULT '',
   gu_email TEXT DEFAULT '',
-  gu_password TEXT,
   gu_enabled INTEGER DEFAULT 1
 );
 
@@ -229,6 +250,13 @@ CREATE TABLE g_user_property (
 );
 CREATE INDEX i_g_user_property_name ON g_user_property(gup_name);
 
+CREATE TABLE g_user_password (
+  guw_id INTEGER PRIMARY KEY AUTOINCREMENT,
+  gu_id INTEGER,
+  guw_password TEXT,
+  FOREIGN KEY(gu_id) REFERENCES g_user(gu_id) ON DELETE CASCADE
+);
+
 CREATE TABLE gpg_code (
   gpgc_id INTEGER PRIMARY KEY AUTOINCREMENT,
   gpgc_plugin_name TEXT NOT NULL,
@@ -337,7 +365,9 @@ CREATE TABLE gpo_code (
   gpoc_redirect_uri TEXT NOT NULL,
   gpoc_code_hash TEXT NOT NULL,
   gpoc_nonce TEXT,
+  gpoc_resource TEXT,
   gpoc_claims_request TEXT DEFAULT NULL,
+  gpoc_authorization_details TEXT DEFAULT NULL,
   gpoc_expires_at TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
   gpoc_issued_for TEXT, -- IP address or hostname
   gpoc_user_agent TEXT,
@@ -368,7 +398,9 @@ CREATE TABLE gpo_refresh_token (
   gpoc_id INTEGER DEFAULT NULL,
   gpor_username TEXT NOT NULL,
   gpor_client_id TEXT,
+  gpor_resource TEXT,
   gpor_claims_request TEXT DEFAULT NULL,
+  gpor_authorization_details TEXT DEFAULT NULL,
   gpor_issued_at TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
   gpor_expires_at TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
   gpor_last_seen TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
@@ -378,6 +410,7 @@ CREATE TABLE gpo_refresh_token (
   gpor_user_agent TEXT,
   gpor_token_hash TEXT NOT NULL,
   gpor_jti TEXT,
+  gpor_dpop_jkt TEXT,
   gpor_enabled INTEGER DEFAULT 1,
   FOREIGN KEY(gpoc_id) REFERENCES gpo_code(gpoc_id) ON DELETE CASCADE
 );
@@ -399,11 +432,13 @@ CREATE TABLE gpo_access_token (
   gpor_id INTEGER DEFAULT NULL,
   gpoa_username TEXT,
   gpoa_client_id TEXT,
+  gpoa_resource TEXT,
   gpoa_issued_at TIMESTAMP DEFAULT CURRENT_TIMESTAMP,
   gpoa_issued_for TEXT, -- IP address or hostname
   gpoa_user_agent TEXT,
   gpoa_token_hash TEXT NOT NULL,
   gpoa_jti TEXT,
+  gpoa_authorization_details TEXT DEFAULT NULL,
   gpoa_enabled INTEGER DEFAULT 1,
   FOREIGN KEY(gpor_id) REFERENCES gpo_refresh_token(gpor_id) ON DELETE CASCADE
 );
@@ -472,6 +507,7 @@ CREATE TABLE gpo_device_authorization (
   gpoda_id INTEGER PRIMARY KEY AUTOINCREMENT,
   gpoda_plugin_name TEXT NOT NULL,
   gpoda_client_id TEXT NOT NULL,
+  gpoda_resource TEXT,
   gpoda_username TEXT,
   gpoda_created_at TIMESTAMP DEFAULT CURRENT_TIMESTAMP,
   gpoda_expires_at TIMESTAMP DEFAULT CURRENT_TIMESTAMP,
@@ -479,6 +515,7 @@ CREATE TABLE gpo_device_authorization (
   gpoda_device_code_hash TEXT NOT NULL,
   gpoda_user_code_hash TEXT NOT NULL,
   gpoda_status INTEGER DEFAULT 0, -- 0: created, 1: user verified, 2 device completed, 3 disabled
+  gpoda_authorization_details TEXT DEFAULT NULL,
   gpoda_last_check TIMESTAMP DEFAULT CURRENT_TIMESTAMP
 );
 CREATE INDEX i_gpoda_device_code_hash ON gpo_device_authorization(gpoda_device_code_hash);
@@ -499,6 +536,62 @@ CREATE TABLE gpo_device_scheme (
   FOREIGN KEY(gpoda_id) REFERENCES gpo_device_authorization(gpoda_id) ON DELETE CASCADE
 );
 
+CREATE TABLE gpo_dpop (
+  gpod_id INTEGER PRIMARY KEY AUTOINCREMENT,
+  gpod_plugin_name TEXT NOT NULL,
+  gpod_client_id TEXT NOT NULL,
+  gpod_jti_hash TEXT NOT NULL,
+  gpod_jkt TEXT NOT NULL,
+  gpod_htm TEXT NOT NULL,
+  gpod_htu TEXT NOT NULL,
+  gpod_iat TIMESTAMP NOT NULL,
+  gpod_last_seen TIMESTAMP DEFAULT CURRENT_TIMESTAMP
+);
+CREATE INDEX i_gpod_jti_hash ON gpo_dpop(gpod_jti_hash);
+
+CREATE TABLE gpo_rar (
+  gporar_id INTEGER PRIMARY KEY AUTOINCREMENT,
+  gporar_plugin_name TEXT NOT NULL,
+  gporar_client_id TEXT NOT NULL,
+  gporar_type TEXT NOT NULL,
+  gporar_username TEXT,
+  gporar_consent INTEGER DEFAULT 0,
+  gporar_enabled INTEGER DEFAULT 1
+);
+CREATE INDEX i_gporar_client_id ON gpo_rar(gporar_client_id);
+CREATE INDEX i_gporar_type ON gpo_rar(gporar_type);
+CREATE INDEX i_gporar_username ON gpo_rar(gporar_username);
+
+CREATE TABLE gpo_par (
+  gpop_id INTEGER PRIMARY KEY AUTOINCREMENT,
+  gpop_plugin_name TEXT NOT NULL,
+  gpop_response_type TEXT NOT NULL,
+  gpop_state TEXT,
+  gpop_username TEXT,
+  gpop_client_id TEXT NOT NULL,
+  gpop_redirect_uri TEXT NOT NULL,
+  gpop_request_uri_hash TEXT NOT NULL,
+  gpop_nonce TEXT,
+  gpop_code_challenge TEXT,
+  gpop_resource TEXT,
+  gpop_claims_request TEXT DEFAULT NULL,
+  gpop_authorization_details TEXT DEFAULT NULL,
+  gpop_additional_parameters TEXT DEFAULT NULL,
+  gpop_status INTEGER DEFAULT 0, -- 0 created, 1 validated, 2 completed
+  gpop_expires_at TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
+  gpop_issued_for TEXT, -- IP address or hostname
+  gpop_user_agent TEXT
+);
+CREATE INDEX i_gpop_request_uri_hash ON gpo_par(gpop_request_uri_hash);
+CREATE INDEX i_gpop_code_challenge ON gpo_par(gpop_code_challenge);
+
+CREATE TABLE gpo_par_scope (
+  gpops_id INTEGER PRIMARY KEY AUTOINCREMENT,
+  gpop_id INTEGER,
+  gpops_scope TEXT NOT NULL,
+  FOREIGN KEY(gpop_id) REFERENCES gpo_par(gpop_id) ON DELETE CASCADE
+);
+
 CREATE TABLE gs_code (
   gsc_id INTEGER PRIMARY KEY AUTOINCREMENT,
   gsc_mod_name TEXT NOT NULL,
@@ -580,20 +673,6 @@ CREATE TABLE gs_user_certificate (
 CREATE INDEX i_gsuc_username ON gs_user_certificate(gsuc_username);
 CREATE INDEX i_gsuc_x509_certificate_id ON gs_user_certificate(gsuc_x509_certificate_id);
 
-CREATE TABLE gs_user_pkcs12 (
-  gsup_id INTEGER PRIMARY KEY AUTOINCREMENT,
-  gsup_mod_name TEXT NOT NULL,
-  gsup_username TEXT NOT NULL,
-  gsup_x509_certificate_content TEXT DEFAULT NULL,
-  gsup_pkcs12_content TEXT DEFAULT NULL,
-  gsup_pkcs12_password TEXT DEFAULT NULL,
-  gsup_activation TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
-  gsup_expiration TIMESTAMP DEFAULT CURRENT_TIMESTAMP,
-  gsup_host TEXT DEFAULT NULL,
-  gsup_user_agent TEXT DEFAULT NULL
-);
-CREATE INDEX i_gsup_username ON gs_user_pkcs12(gsup_username);
-
 CREATE TABLE gpr_session (
   gprs_id INTEGER PRIMARY KEY AUTOINCREMENT,
   gprs_plugin_name TEXT NOT NULL,
@@ -601,6 +680,7 @@ CREATE TABLE gpr_session (
   gprs_name TEXT,
   gprs_email TEXT,
   gprs_code_hash TEXT,
+  gprs_callback_url TEXT DEFAULT NULL,
   gprs_password_set INTEGER DEFAULT 0,
   gprs_session_hash TEXT,
   gprs_token_hash TEXT,
@@ -613,6 +693,45 @@ CREATE INDEX i_gprs_session_hash ON gpr_session(gprs_session_hash);
 CREATE INDEX i_gprs_gprs_token_hash ON gpr_session(gprs_token_hash);
 CREATE INDEX i_gprs_gprs_gprs_code_hash ON gpr_session(gprs_code_hash);
 
+CREATE TABLE gpr_update_email (
+  gprue_id INTEGER PRIMARY KEY AUTOINCREMENT,
+  gprue_plugin_name TEXT NOT NULL,
+  gprue_username TEXT NOT NULL,
+  gprue_email TEXT,
+  gprue_token_hash TEXT,
+  gprue_expires_at TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
+  gprue_issued_for TEXT, -- IP address or hostname
+  gprue_user_agent TEXT,
+  gprue_enabled INTEGER DEFAULT 1
+);
+CREATE INDEX i_gprue_token_hash ON gpr_update_email(gprue_token_hash);
+
+CREATE TABLE gpr_reset_credentials_session (
+  gprrcs_id INTEGER PRIMARY KEY AUTOINCREMENT,
+  gprrcs_plugin_name TEXT NOT NULL,
+  gprrcs_username TEXT NOT NULL,
+  gprrcs_session_hash TEXT,
+  gprrcs_callback_url TEXT DEFAULT NULL,
+  gprrcs_expires_at TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
+  gprrcs_issued_for TEXT, -- IP address or hostname
+  gprrcs_user_agent TEXT,
+  gprrcs_enabled INTEGER DEFAULT 1
+);
+CREATE INDEX i_gprrcs_session_hash ON gpr_reset_credentials_session(gprrcs_session_hash);
+
+CREATE TABLE gpr_reset_credentials_email (
+  gprrct_id INTEGER PRIMARY KEY AUTOINCREMENT,
+  gprrct_plugin_name TEXT NOT NULL,
+  gprrct_username TEXT NOT NULL,
+  gprrct_token_hash TEXT,
+  gprrct_callback_url TEXT DEFAULT NULL,
+  gprrct_expires_at TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
+  gprrct_issued_for TEXT, -- IP address or hostname
+  gprrct_user_agent TEXT,
+  gprrct_enabled INTEGER DEFAULT 1
+);
+CREATE INDEX i_gprrct_token_hash ON gpr_reset_credentials_email(gprrct_token_hash);
+
 CREATE TABLE gs_oauth2_registration (
   gsor_id INTEGER PRIMARY KEY AUTOINCREMENT,
   gsor_mod_name TEXT NOT NULL,
@@ -637,9 +756,10 @@ CREATE TABLE gs_oauth2_session (
 INSERT INTO g_scope (gs_name, gs_display_name, gs_description, gs_password_required, gs_password_max_age) VALUES ('g_admin', 'Glewlwyd administration', 'Access to Glewlwyd''s administration API', 1, 600);
 INSERT INTO g_scope (gs_name, gs_display_name, gs_description, gs_password_required, gs_password_max_age) VALUES ('g_profile', 'Glewlwyd profile', 'Access to the user''s profile API', 1, 600);
 INSERT INTO g_scope (gs_name, gs_display_name, gs_description, gs_password_required, gs_password_max_age) VALUES ('openid', 'Open ID', 'Open ID Connect scope', 0, 0);
-INSERT INTO g_user_module_instance (gumi_module, gumi_order, gumi_name, gumi_display_name, gumi_parameters, gumi_readonly) VALUES ('database', 0, 'database', 'Database backend', '{"use-glewlwyd-connection":true,"data-format":{"picture":{"multiple":false,"read":true,"write":true,"profile-read":true,"profile-write":true}}}', 0);
-INSERT INTO g_client_module_instance (gcmi_module, gcmi_order, gcmi_name, gcmi_display_name, gcmi_parameters, gcmi_readonly) VALUES ('database', 0, 'database', 'Database backend', '{"use-glewlwyd-connection":true,"data-format":{"redirect_uri":{"multiple":true,"read":true,"write":true},"authorization_type":{"multiple":true,"read":true,"write":true},"client_secret":{"read":true,"write":true},"pubkey":{"read":true,"write":true},"jwks":{"convert":"jwks","read":true,"write":true},"jwks_uri":{"read":true,"write":true},"post_logout_redirect_uris":{"multiple":true,"read":true,"write":true},"token_endpoint_auth_method":{"multiple":false,"read":true,"write":true}}}', 0);
-INSERT INTO g_user (gu_username, gu_name, gu_password, gu_email, gu_enabled) VALUES ('admin', 'The Administrator', 'fOfvZC/wR2cUSTWbW6YZueGyyDuFqwkoFlcNlRYWJscxYTVOVFJ3VWFHdVJQT0pU', '', 1);
+INSERT INTO g_user_module_instance (gumi_module, gumi_order, gumi_name, gumi_display_name, gumi_parameters, gumi_readonly) VALUES ('database', 0, 'database', 'Database backend', '{"use-glewlwyd-connection":true,"data-format":{"picture":{"multiple":false,"read":true,"write":true,"profile-read":true,"profile-write":true},"reset-credentials-code":{"multiple":false,"read":true,"write":true,"profile-read":false,"profile-write":false}}}', 0);
+INSERT INTO g_client_module_instance (gcmi_module, gcmi_order, gcmi_name, gcmi_display_name, gcmi_parameters, gcmi_readonly) VALUES ('database', 0, 'database', 'Database backend', '{"use-glewlwyd-connection":true,"data-format":{"redirect_uri":{"multiple":true,"read":true,"write":true},"authorization_type":{"multiple":true,"read":true,"write":true},"client_secret":{"read":true,"write":true},"pubkey":{"read":true,"write":true},"jwks":{"convert":"jwks","read":true,"write":true},"jwks_uri":{"read":true,"write":true},"post_logout_redirect_uris":{"multiple":true,"read":true,"write":true},"token_endpoint_auth_method":{"multiple":false,"read":true,"write":true},"resource":{"multiple":true,"read":true,"write":true},"authorization_data_types":{"multiple":true,"read":true,"write":true}}}', 0);
+INSERT INTO g_user (gu_username, gu_name, gu_email, gu_enabled) VALUES ('admin', 'The Administrator', '', 1);
+INSERT INTO g_user_password (gu_id, guw_password) VALUES ((SELECT gu_id from g_user WHERE gu_username='admin'), 'fOfvZC/wR2cUSTWbW6YZueGyyDuFqwkoFlcNlRYWJscxYTVOVFJ3VWFHdVJQT0pU');
 INSERT INTO g_user_scope (gus_name) VALUES ('g_admin');
 INSERT INTO g_user_scope (gus_name) VALUES ('g_profile');
 INSERT INTO g_user_scope_user (gu_id, gus_id) VALUES ((SELECT gu_id from g_user WHERE gu_username='admin'), (SELECT gus_id FROM g_user_scope WHERE gus_name='g_admin'));
