@@ -1069,177 +1069,11 @@ static char * get_client_dn_from_client_id(json_t * j_params, LDAP * ldap, const
 
 json_t * client_module_load(struct config_module * config) {
   UNUSED(config);
-  return json_pack("{si ss ss ss s{ s{ssso} s{ssso} s{ssso} s{ssso} s{ssso} s{ssso} s{ssso} s{ssso} s{ssso} s[{s{ssso} s{ssso} s{sssos[ssss]}}] s{ssso} s{ssso} s{ssso} s{ssso} s{ssso} s{sssos[sssss]} s{ssso} s{s{s{ssso} s{ssso} s{ssso} s{ssso} s{ssso}}}}}",
-                   "result",
-                   G_OK,
-                   
-                   "name",
-                   "ldap",
-                   
-                   "display_name",
-                   "LDAP backend client module",
-                   
-                   "description",
-                   "Module to store clients in a LDAP server",
-                   
-                   "parameters",
-                     "uri",
-                       "type",
-                       "string",
-                       "mandatory",
-                       json_true(),
-                       
-                     "bind-dn",
-                       "type",
-                       "string",
-                       "mandatory",
-                       json_true(),
-                       
-                     "bind-password",
-                       "type",
-                       "string",
-                       "mandatory",
-                       json_true(),
-                       
-                     "base-search",
-                       "type",
-                       "string",
-                       "mandatory",
-                       json_true(),
-                       
-                     "filter",
-                       "type",
-                       "string",
-                       "mandatory",
-                       json_true(),
-                       
-                     "client_id-property",
-                       "type",
-                       "string",
-                       "mandatory",
-                       json_true(),
-                       
-                     "scope-property",
-                       "type",
-                       "string",
-                       "mandatory",
-                       json_true(),
-                       
-                     "page-size",
-                       "type",
-                       "number",
-                       "mandatory",
-                       json_false(),
-                       
-                     "search-scope",
-                       "type",
-                       "string",
-                       "mandatory",
-                       json_false(),
-                       
-                     "scope-match",
-                       "ldap-value",
-                         "type",
-                         "string",
-                         "mandatory",
-                         json_true(),
-                         
-                       "scope-value",
-                         "type",
-                         "string",
-                         "mandatory",
-                         json_true(),
-                         
-                       "match",
-                         "type",
-                         "list",
-                         "mandatory",
-                         json_true(),
-                         "values",
-                           "equals",
-                           "contains",
-                           "startswith",
-                           "endswith",
-                           
-                     "name-property",
-                       "type",
-                       "string",
-                       "mandatory",
-                       json_false(),
-                       
-                     "description-property",
-                       "type",
-                       "string",
-                       "mandatory",
-                       json_false(),
-                       
-                     "confidential-property",
-                       "type",
-                       "string",
-                       "mandatory",
-                       json_false(),
-                       
-                     "rdn-property",
-                       "type",
-                       "string",
-                       "mandatory",
-                       json_false(),
-                       
-                     "password-property",
-                       "type",
-                       "string",
-                       "mandatory",
-                       json_false(),
-                       
-                     "password-algorithm",
-                       "type",
-                       "list",
-                       "mandatory",
-                       json_false(),
-                       "values",
-                         "SSHA",
-                         "SHA",
-                         "SMS5",
-                         "MD5",
-                         "PLAIN",
-                         
-                     "object-class",
-                       "type",
-                       "string",
-                       "mandatory",
-                       json_false(),
-                       
-                     "data-format",
-                       "field-name",
-                         "multiple",
-                           "type",
-                           "boolean",
-                           "default",
-                           json_false(),
-                           
-                         "read",
-                           "type",
-                           "boolean",
-                           "default",
-                           json_true(),
-                           
-                         "write",
-                           "type",
-                           "boolean",
-                           "default",
-                           json_true(),
-                           
-                         "profile-read",
-                           "type",
-                           "boolean",
-                           "default",
-                           json_false(),
-                           
-                         "profile-write",
-                           "type",
-                           "boolean",
-                           "default",
-                           json_false());
+  return json_pack("{si ss ss ss}",
+                   "result", G_OK,
+                   "name", "ldap",
+                   "display_name", "LDAP backend client module",
+                   "description", "Module to store clients in a LDAP server");
 }
 
 int client_module_unload(struct config_module * config) {
