@@ -8,7 +8,7 @@
  * 
  * Declarations for common constants and prototypes used in Glewlwyd main program and modules
  *
- * Copyright 2016-2020 Nicolas Mora <mail@babelouest.org>
+ * Copyright 2016-2021 Nicolas Mora <mail@babelouest.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU GENERAL PUBLIC LICENSE
@@ -37,7 +37,8 @@
 #include <gnutls/gnutls.h>
 #include <gnutls/x509.h>
 
-#include "static_file_callback.h"
+#include "static_compressed_inmemory_website_callback.h"
+#include "http_compression_callback.h"
 
 /**
  * Result values used in the application
@@ -56,8 +57,9 @@
 #define GLEWLWYD_CALLBACK_PRIORITY_ZERO           0
 #define GLEWLWYD_CALLBACK_PRIORITY_AUTHENTICATION 1
 #define GLEWLWYD_CALLBACK_PRIORITY_APPLICATION    2
-#define GLEWLWYD_CALLBACK_PRIORITY_CLOSE          3
-#define GLEWLWYD_CALLBACK_PRIORITY_PLUGIN         4
+#define GLEWLWYD_CALLBACK_PRIORITY_COMPRESSION    3
+#define GLEWLWYD_CALLBACK_PRIORITY_CLOSE          4
+#define GLEWLWYD_CALLBACK_PRIORITY_PLUGIN         5
 #define GLEWLWYD_CALLBACK_PRIORITY_FILE           100
 
 /**
@@ -133,10 +135,10 @@ struct _user_module {
   char      * name;
   char      * display_name;
   char      * description;
-  json_t    * parameters;
+  double      api_version;
   json_t * (* user_module_load)(struct config_module * config);
   int      (* user_module_unload)(struct config_module * config);
-  json_t * (* user_module_init)(struct config_module * config, int readonly, json_t * j_parameters, void ** cls);
+  json_t * (* user_module_init)(struct config_module * config, int readonly, int multiple_passwords, json_t * j_parameters, void ** cls);
   int      (* user_module_close)(struct config_module * config, void * cls);
   size_t   (* user_module_count_total)(struct config_module * config, const char * pattern, void * cls);
   json_t * (* user_module_get_list)(struct config_module * config, const char * pattern, size_t offset, size_t limit, void * cls);
@@ -148,7 +150,7 @@ struct _user_module {
   int      (* user_module_update_profile)(struct config_module * config, const char * username, json_t * j_user, void * cls);
   int      (* user_module_delete)(struct config_module * config, const char * username, void * cls);
   int      (* user_module_check_password)(struct config_module * config, const char * username, const char * password, void * cls);
-  int      (* user_module_update_password)(struct config_module * config, const char * username, const char * new_password, void * cls);
+  int      (* user_module_update_password)(struct config_module * config, const char * username, const char ** new_passwords, size_t new_passwords_len, void * cls);
   };
 
 /**
@@ -160,6 +162,7 @@ struct _user_module_instance {
   void                * cls;
   short int             enabled;
   short int             readonly;
+  short int             multiple_passwords;
 };
 
 /**
@@ -170,7 +173,7 @@ struct _client_module {
   char     * name;
   char     * display_name;
   char     * description;
-  json_t   * parameters;
+  double     api_version;
   json_t * (* client_module_load)(struct config_module * config);
   int      (* client_module_unload)(struct config_module * config);
   json_t * (* client_module_init)(struct config_module * config, int readonly, json_t * j_parameters, void ** cls);
@@ -204,17 +207,18 @@ struct _user_auth_scheme_module {
   char       * name;
   char       * display_name;
   char       * description;
-  json_t     * parameters;
+  double       api_version;
   json_t  * (* user_auth_scheme_module_load)(struct config_module * config);
   int       (* user_auth_scheme_module_unload)(struct config_module * config);
   json_t *  (* user_auth_scheme_module_init)(struct config_module * config, json_t * j_parameters, const char * mod_name, void ** cls);
   int       (* user_auth_scheme_module_close)(struct config_module * config, void * cls);
   int       (* user_auth_scheme_module_can_use)(struct config_module * config, const char * username, void * cls);
-  json_t  * (* user_auth_scheme_module_register)(struct config_module * config, const void * http_request, const char * username, json_t * j_scheme_data, void * cls);
-  json_t  * (* user_auth_scheme_module_register_get)(struct config_module * config, const void * http_request, const char * username, void * cls);
+  json_t  * (* user_auth_scheme_module_register)(struct config_module * config, const struct _u_request * http_request, const char * username, json_t * j_scheme_data, void * cls);
+  json_t  * (* user_auth_scheme_module_register_get)(struct config_module * config, const struct _u_request * http_request, const char * username, void * cls);
   int       (* user_auth_scheme_module_deregister)(struct config_module * config, const char * username, void * cls);
-  json_t  * (* user_auth_scheme_module_trigger)(struct config_module * config, const void * http_request, const char * username, json_t * j_scheme_trigger, void * cls);
-  int       (* user_auth_scheme_module_validate)(struct config_module * config, const void * http_request, const char * username, json_t * j_scheme_data, void * cls);
+  json_t  * (* user_auth_scheme_module_trigger)(struct config_module * config, const struct _u_request * http_request, const char * username, json_t * j_scheme_trigger, void * cls);
+  int       (* user_auth_scheme_module_validate)(struct config_module * config, const struct _u_request * http_request, const char * username, json_t * j_scheme_data, void * cls);
+  json_t *  (* user_auth_scheme_module_identify)(struct config_module * config, const struct _u_request * http_request, json_t * j_scheme_data, void * cls);
 };
 
 /**
@@ -241,7 +245,7 @@ struct _plugin_module {
   char      * name;
   char      * display_name;
   char      * description;
-  json_t    * parameters;
+  double      api_version;
   json_t * (* plugin_module_load)(struct config_plugin * config);
   int      (* plugin_module_unload)(struct config_plugin * config);
   json_t * (* plugin_module_init)(struct config_plugin * config, const char * name, json_t * j_parameters, void ** cls);
@@ -262,49 +266,49 @@ struct _plugin_module_instance {
  * Structure used to store the global application config
  */
 struct config_elements {
-  char *                                      config_file;
-  unsigned int                                port;
-  char *                                      bind_address;
-  char *                                      external_url;
-  char *                                      api_prefix;
-  char *                                      cookie_domain;
-  unsigned int                                cookie_secure;
-  unsigned short                              log_mode_args;
-  unsigned short                              log_level_args;
-  unsigned long                               log_mode;
-  unsigned long                               log_level;
-  char *                                      log_file;
-  struct _static_file_config *                static_file_config;
-  char *                                      admin_scope;
-  char *                                      profile_scope;
-  char *                                      allow_origin;
-  unsigned int                                use_secure_connection;
-  char *                                      secure_connection_key_file;
-  char *                                      secure_connection_pem_file;
-  char *                                      secure_connection_ca_file;
-  struct _h_connection *                      conn;
-  struct _u_instance *                        instance;
-  unsigned int                                instance_initialized;
-  char *                                      session_key;
-  unsigned int                                session_expiration;
-  unsigned int                                salt_length;
-  digest_algorithm                            hash_algorithm;
-  char *                                      login_url;
-  unsigned int                                delete_profile;
-  char *                                      user_module_path;
-  struct _pointer_list *                      user_module_list;
-  struct _pointer_list *                      user_module_instance_list;
-  char *                                      client_module_path;
-  struct _pointer_list *                      client_module_list;
-  struct _pointer_list *                      client_module_instance_list;
-  char *                                      user_auth_scheme_module_path;
-  struct _pointer_list *                      user_auth_scheme_module_list;
-  struct _pointer_list *                      user_auth_scheme_module_instance_list;
-  char *                                      plugin_module_path;
-  struct _pointer_list *                      plugin_module_list;
-  struct _pointer_list *                      plugin_module_instance_list;
-  struct config_plugin *                      config_p;
-  struct config_module *                      config_m;
+  char *                                         config_file;
+  unsigned int                                   port;
+  char *                                         bind_address;
+  char *                                         external_url;
+  char *                                         api_prefix;
+  char *                                         cookie_domain;
+  unsigned int                                   cookie_secure;
+  unsigned short                                 log_mode_args;
+  unsigned short                                 log_level_args;
+  unsigned long                                  log_mode;
+  unsigned long                                  log_level;
+  char *                                         log_file;
+  struct _u_compressed_inmemory_website_config * static_file_config;
+  char *                                         admin_scope;
+  char *                                         profile_scope;
+  char *                                         allow_origin;
+  unsigned int                                   use_secure_connection;
+  char *                                         secure_connection_key_file;
+  char *                                         secure_connection_pem_file;
+  char *                                         secure_connection_ca_file;
+  struct _h_connection *                         conn;
+  struct _u_instance *                           instance;
+  unsigned int                                   instance_initialized;
+  char *                                         session_key;
+  unsigned int                                   session_expiration;
+  unsigned int                                   salt_length;
+  digest_algorithm                               hash_algorithm;
+  char *                                         login_url;
+  unsigned int                                   delete_profile;
+  char *                                         user_module_path;
+  struct _pointer_list *                         user_module_list;
+  struct _pointer_list *                         user_module_instance_list;
+  char *                                         client_module_path;
+  struct _pointer_list *                         client_module_list;
+  struct _pointer_list *                         client_module_instance_list;
+  char *                                         user_auth_scheme_module_path;
+  struct _pointer_list *                         user_auth_scheme_module_list;
+  struct _pointer_list *                         user_auth_scheme_module_instance_list;
+  char *                                         plugin_module_path;
+  struct _pointer_list *                         plugin_module_list;
+  struct _pointer_list *                         plugin_module_instance_list;
+  struct config_plugin *                         config_p;
+  struct config_module *                         config_m;
 };
 
 /**
@@ -345,6 +349,7 @@ struct config_plugin {
   int      (* glewlwyd_plugin_callback_delete_client)(struct config_plugin * config, const char * client_id);
 
   // Register scheme functions
+  json_t * (* glewlwyd_plugin_callback_get_scheme_list)(struct config_plugin * config, const char * username);
   json_t * (* glewlwyd_plugin_callback_scheme_register)(struct config_plugin * config, const char * mod_name, const struct _u_request * http_request, const char * username, json_t * j_scheme_data);
   json_t * (* glewlwyd_plugin_callback_scheme_register_get)(struct config_plugin * config, const char * mod_name, const struct _u_request * http_request, const char * username);
   int      (* glewlwyd_plugin_callback_scheme_deregister)(struct config_plugin * config, const char * mod_name, const char * username);
@@ -405,7 +410,7 @@ int check_result_value(json_t * result, const int value);
  */
 json_t * user_module_load(struct config_module * config);
 int      user_module_unload(struct config_module * config);
-json_t * user_module_init(struct config_module * config, int readonly, json_t * j_parameters, void ** cls);
+json_t * user_module_init(struct config_module * config, int readonly, int multiple_passwords, json_t * j_parameters, void ** cls);
 int      user_module_close(struct config_module * config, void * cls);
 size_t   user_module_count_total(struct config_module * config, const char * pattern, void * cls);
 json_t * user_module_get_list(struct config_module * config, const char * pattern, size_t offset, size_t limit, void * cls);
@@ -417,7 +422,7 @@ int      user_module_update(struct config_module * config, const char * username
 int      user_module_update_profile(struct config_module * config, const char * username, json_t * j_user, void * cls);
 int      user_module_delete(struct config_module * config, const char * username, void * cls);
 int      user_module_check_password(struct config_module * config, const char * username, const char * password, void * cls);
-int      user_module_update_password(struct config_module * config, const char * username, const char * new_password, void * cls);
+int      user_module_update_password(struct config_module * config, const char * username, const char ** new_passwords, size_t new_passwords_len, void * cls);
 
 /**
  * Client functions prototypes
@@ -448,6 +453,7 @@ json_t * user_auth_scheme_module_register_get(struct config_module * config, con
 int      user_auth_scheme_module_deregister(struct config_module * config, const char * username, void * cls);
 json_t * user_auth_scheme_module_trigger(struct config_module * config, const struct _u_request * http_request, const char * username, json_t * j_scheme_trigger, void * cls);
 int      user_auth_scheme_module_validate(struct config_module * config, const struct _u_request * http_request, const char * username, json_t * j_scheme_data, void * cls);
+json_t * user_auth_scheme_module_identify(struct config_module * config, const struct _u_request * http_request, json_t * j_scheme_data, void * cls);
 
 /**
  * Plugin functions prototypes
