@@ -8,7 +8,7 @@
  * 
  * Declarations for constants and prototypes
  *
- * Copyright 2016-2020 Nicolas Mora <mail@babelouest.org>
+ * Copyright 2016-2021 Nicolas Mora <mail@babelouest.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU GENERAL PUBLIC LICENSE
@@ -28,7 +28,7 @@
 #ifndef __GLEWLWYD_H_
 #define __GLEWLWYD_H_
 
-#define _GLEWLWYD_VERSION_ "2.3.3"
+#define _GLEWLWYD_VERSION_ "2.5.2"
 
 #include <jansson.h>
 
@@ -55,6 +55,8 @@
 
 #define GLEWLWYD_LOG_NAME "Glewlwyd"
 
+#define _GLEWLWYD_USER_MODULE_VERSION 2.5
+
 // Configuration default values
 #define GLEWLWYD_DEFAULT_PORT                              4593
 #define GLEWLWYD_DEFAULT_API_PREFIX                        "api"
@@ -69,6 +71,9 @@
 #define GLEWLWYD_DEFAULT_SESSION_EXPIRATION_PASSWORD       40320   // 4 weeks
 #define GLEWLWYD_RESET_PASSWORD_DEFAULT_SESSION_EXPIRATION 2592000 // 30 days
 #define GLEWLWYD_SESSION_ID_LENGTH                         128
+#define GLEWLWYD_API_KEY_HEADER_KEY                        "Authorization"
+#define GLEWLWYD_API_KEY_HEADER_PREFIX                     "token "
+#define GLEWLWYD_API_KEY_LENGTH                            32
 
 #define GLEWLWYD_RUNNING     0
 #define GLEWLWYD_STOP        1
@@ -85,6 +90,7 @@
 #define GLEWLWYD_TABLE_SCOPE_GROUP                             "g_scope_group"
 #define GLEWLWYD_TABLE_SCOPE_GROUP_AUTH_SCHEME_MODULE_INSTANCE "g_scope_group_auth_scheme_module_instance"
 #define GLEWLWYD_TABLE_CLIENT_USER_SCOPE                       "g_client_user_scope"
+#define GLEWLWYD_TABLE_API_KEY                                 "g_api_key"
 
 // Module management
 #define GLEWLWYD_MODULE_ACTION_STOP  0
@@ -111,7 +117,7 @@
 #define GLEWLWYD_ENV_PROFILE_SCOPE              "GLWD_PROFILE_SCOPE"
 #define GLEWLWYD_ENV_USER_MODULE_PATH           "GLWD_USER_MODULE_PATH"
 #define GLEWLWYD_ENV_CLIENT_MODULE_PATH         "GLWD_CLIENT_MODULE_PATH"
-#define GLEWLWYD_ENV_AUTH_SCHEME_MODUE_PATH     "GLWD_AUTH_SCHEME_MODUE_PATH"
+#define GLEWLWYD_ENV_AUTH_SCHEME_MODULE_PATH    "GLWD_AUTH_SCHEME_MODULE_PATH"
 #define GLEWLWYD_ENV_PLUGIN_MODULE_PATH         "GLWD_PLUGIN_MODULE_PATH"
 #define GLEWLWYD_ENV_USE_SECURE_CONNECTION      "GLWD_USE_SECURE_CONNECTION"
 #define GLEWLWYD_ENV_SECURE_CONNECTION_KEY_FILE "GLWD_SECURE_CONNECTION_KEY_FILE"
@@ -160,12 +166,14 @@ int module_instance_parameters_check(const char * module_parameters, const char
 // Validate user login/password credentials
 json_t * auth_check_user_credentials(struct config_elements * config, const char * username, const char * password);
 json_t * auth_check_user_scheme(struct config_elements * config, const char * scheme_type, const char * scheme_name, const char * username, json_t * scheme_parameters, const struct _u_request * request);
+json_t * auth_check_identify_scheme(struct config_elements * config, const char * scheme_type, const char * scheme_name, json_t * j_scheme_value, const struct _u_request * request);
 json_t * auth_register_user_scheme(struct config_elements * config, const char * scheme_type, const char * scheme_name, const char * username, int delegate, json_t * j_register_parameters, const struct _u_request * request);
 json_t * auth_register_get_user_scheme(struct config_elements * config, const char * scheme_type, const char * scheme_name, const char * username, const struct _u_request * request);
 json_t * auth_trigger_user_scheme(struct config_elements * config, const char * scheme_type, const char * scheme_name, const char * username, json_t * register_parameters, const struct _u_request * request);
+json_t * auth_trigger_identify_scheme(struct config_elements * config, const char * scheme_type, const char * scheme_name, json_t * j_trigger_parameters, const struct _u_request * request);
 
 // Session
-int user_session_update(struct config_elements * config, const char * session_uid, const char * user_agent, const char * issued_for, const char * username, const char * scheme_name);
+int user_session_update(struct config_elements * config, const char * session_uid, const char * user_agent, const char * issued_for, const char * username, const char * scheme_name, int update_login);
 json_t * get_session_for_username(struct config_elements * config, const char * session_uid, const char * username);
 json_t * get_current_user_for_session(struct config_elements * config, const char * session_uid);
 json_t * get_users_for_session(struct config_elements * config, const char * session_uid);
@@ -179,12 +187,13 @@ int delete_user_session_from_hash(struct config_elements * config, const char *
 json_t * user_set_profile(struct config_elements * config, const char * username, json_t * j_profile);
 int user_delete_profile(struct config_elements * config, const char * username);
 json_t * user_get_profile(struct config_elements * config, const char * username);
-int user_update_password(struct config_elements * config, const char * username, const char * old_password, const char * new_password);
-int user_set_password(struct config_elements * config, const char * username, const char * new_password);
+int user_update_password(struct config_elements * config, const char * username, const char * old_password, const char ** new_passwords, size_t new_passwords_len);
+int user_set_password(struct config_elements * config, const char * username, const char ** new_passwords, size_t new_passwords_len);
 json_t * get_scheme_list_for_user(struct config_elements * config, const char * username);
 
 // User
 int user_has_scope(json_t * j_user, const char * scope);
+int user_has_scheme(struct config_elements * config, const char * username, const char * scheme_name);
 
 // Client
 json_t * auth_check_client_credentials(struct config_elements * config, const char * client_id, const char * password);
@@ -210,6 +219,8 @@ json_t * add_user_module(struct config_elements * config, json_t * j_module);
 int set_user_module(struct config_elements * config, const char * name, json_t * j_module);
 int delete_user_module(struct config_elements * config, const char * name);
 json_t * manage_user_module(struct config_elements * config, const char * name, int action);
+void close_user_module_instance_list(struct config_elements * config);
+void close_user_module_list(struct config_elements * config);
 
 // User auth scheme module functions
 json_t * get_user_auth_scheme_module_list(struct config_elements * config);
@@ -219,6 +230,8 @@ json_t * add_user_auth_scheme_module(struct config_elements * config, json_t * j
 int set_user_auth_scheme_module(struct config_elements * config, const char * name, json_t * j_module);
 int delete_user_auth_scheme_module(struct config_elements * config, const char * name);
 json_t * manage_user_auth_scheme_module(struct config_elements * config, const char * name, int action);
+void close_user_auth_scheme_module_instance_list(struct config_elements * config);
+void close_user_auth_scheme_module_list(struct config_elements * config);
 
 // Client module functions
 json_t * get_client_module_list(struct config_elements * config);
@@ -228,6 +241,8 @@ json_t * add_client_module(struct config_elements * config, json_t * j_module);
 int set_client_module(struct config_elements * config, const char * name, json_t * j_module);
 int delete_client_module(struct config_elements * config, const char * name);
 json_t * manage_client_module(struct config_elements * config, const char * name, int action);
+void close_client_module_instance_list(struct config_elements * config);
+void close_client_module_list(struct config_elements * config);
 
 // Plugin module functions
 json_t * get_plugin_module_list_for_user(struct config_elements * config);
@@ -238,6 +253,8 @@ json_t * add_plugin_module(struct config_elements * config, json_t * j_module);
 int set_plugin_module(struct config_elements * config, const char * name, json_t * j_module);
 int delete_plugin_module(struct config_elements * config, const char * name);
 json_t * manage_plugin_module(struct config_elements * config, const char * name, int action);
+void close_plugin_module_instance_list(struct config_elements * config);
+void close_plugin_module_list(struct config_elements * config);
 
 // Plugin functions
 int glewlwyd_callback_add_plugin_endpoint(struct config_plugin * config, const char * method, const char * name, const char * url, unsigned int priority, int (* callback)(const struct _u_request * request, struct _u_response * response, void * user_data), void * user_data);
@@ -265,6 +282,7 @@ json_t * glewlwyd_plugin_callback_is_client_valid(struct config_plugin * config,
 int glewlwyd_plugin_callback_add_client(struct config_plugin * config, json_t * j_client);
 int glewlwyd_plugin_callback_set_client(struct config_plugin * config, const char * client_id, json_t * j_client);
 int glewlwyd_plugin_callback_delete_client(struct config_plugin * config, const char * client_id);
+json_t * glewlwyd_plugin_callback_get_scheme_list(struct config_plugin * config, const char * username);
 json_t * glewlwyd_plugin_callback_scheme_register(struct config_plugin * config, const char * mod_name, const struct _u_request * http_request, const char * username, json_t * j_scheme_data);
 json_t * glewlwyd_plugin_callback_scheme_register_get(struct config_plugin * config, const char * mod_name, const struct _u_request * http_request, const char * username);
 int glewlwyd_plugin_callback_scheme_can_use(struct config_plugin * config, const char * mod_name, const char * username);
@@ -299,10 +317,16 @@ int add_scope(struct config_elements * config, json_t * j_scope);
 int set_scope(struct config_elements * config, const char * scope, json_t * j_scope);
 int delete_scope(struct config_elements * config, const char * scope);
 
-// Callback functions
+// API key CRD functions
+int verify_api_key(struct config_elements * config, const char * api_key);
+json_t * get_api_key_list(struct config_elements * config, const char * pattern, size_t offset, size_t limit);
+json_t * generate_api_key(struct config_elements * config, const char * username, const char * issued_for, const char * user_agent);
+int disable_api_key(struct config_elements * config, const char * token_hash);
 
+// Callback functions
 int callback_glewlwyd_check_user_session (const struct _u_request * request, struct _u_response * response, void * user_data);
 int callback_glewlwyd_check_admin_session (const struct _u_request * request, struct _u_response * response, void * user_data);
+int callback_glewlwyd_check_admin_session_or_api_key (const struct _u_request * request, struct _u_response * response, void * user_data);
 int callback_glewlwyd_check_admin_session_delegate (const struct _u_request * request, struct _u_response * response, void * user_data);
 int callback_glewlwyd_close_check_session (const struct _u_request * request, struct _u_response * response, void * user_data);
 int callback_glewlwyd_check_user_profile_valid (const struct _u_request * request, struct _u_response * response, void * user_data);
@@ -333,6 +357,7 @@ int callback_glewlwyd_options (const struct _u_request * request, struct _u_resp
 int callback_glewlwyd_server_configuration (const struct _u_request * request, struct _u_response * response, void * user_data);
 
 int callback_glewlwyd_get_module_type_list (const struct _u_request * request, struct _u_response * response, void * user_data);
+int callback_glewlwyd_reload_modules (const struct _u_request * request, struct _u_response * response, void * user_data);
 
 int callback_glewlwyd_get_user_module_list (const struct _u_request * request, struct _u_response * response, void * user_data);
 int callback_glewlwyd_get_user_module (const struct _u_request * request, struct _u_response * response, void * user_data);
@@ -380,6 +405,10 @@ int callback_glewlwyd_add_scope (const struct _u_request * request, struct _u_re
 int callback_glewlwyd_set_scope (const struct _u_request * request, struct _u_response * response, void * plugin_data);
 int callback_glewlwyd_delete_scope (const struct _u_request * request, struct _u_response * response, void * plugin_data);
 
+int callback_glewlwyd_get_api_key_list (const struct _u_request * request, struct _u_response * response, void * plugin_data);
+int callback_glewlwyd_delete_api_key (const struct _u_request * request, struct _u_response * response, void * plugin_data);
+int callback_glewlwyd_add_api_key (const struct _u_request * request, struct _u_response * response, void * plugin_data);
+
 int callback_default (const struct _u_request * request, struct _u_response * response, void * user_data);
 
 #endif
