@@ -8,7 +8,7 @@
  * 
  * Definitions for functions used in plugin modules
  * 
- * Copyright 2016-2020 Nicolas Mora <mail@babelouest.org>
+ * Copyright 2016-2021 Nicolas Mora <mail@babelouest.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU GENERAL PUBLIC LICENSE
@@ -96,7 +96,7 @@ json_t * glewlwyd_callback_check_session_valid(struct config_plugin * config, co
         if (check_result_value(j_scope_allowed, G_OK)) {
           j_return = json_pack("{sis{sOsO}}", "result", G_OK, "session", "scope", json_object_get(j_scope_allowed, "scheme"), "user", json_object_get(j_user, "user"));
         } else if (check_result_value(j_scope_allowed, G_ERROR_UNAUTHORIZED) || check_result_value(j_scope_allowed, G_ERROR_NOT_FOUND)) {
-          j_return = json_pack("{si}", "result", G_ERROR_UNAUTHORIZED);
+          j_return = json_pack("{sis{sO}}", "result", G_ERROR_UNAUTHORIZED, "session", "user", json_object_get(j_user, "user"));
         } else {
           y_log_message(Y_LOG_LEVEL_ERROR, "glewlwyd_callback_check_session_valid - Error get_validated_auth_scheme_list_from_scope_list");
           j_return = json_pack("{si}", "result", G_ERROR);
@@ -106,7 +106,7 @@ json_t * glewlwyd_callback_check_session_valid(struct config_plugin * config, co
         j_return = json_pack("{sis{sO}}", "result", G_OK, "session", "user", json_object_get(j_user, "user"));
       }
     } else if (check_result_value(j_user, G_ERROR_NOT_FOUND)) {
-      j_return = json_pack("{si}", "result", G_ERROR_UNAUTHORIZED);
+      j_return = json_pack("{si}", "result", G_ERROR_NOT_FOUND);
     } else {
       y_log_message(Y_LOG_LEVEL_ERROR, "glewlwyd_callback_check_session_valid - Error get_current_user_for_session");
       j_return = json_pack("{si}", "result", G_ERROR);
@@ -491,9 +491,11 @@ int glewlwyd_plugin_callback_set_user(struct config_plugin * config, const char
 int glewlwyd_plugin_callback_user_update_password(struct config_plugin * config, const char * username, const char * password) {
   json_t * j_user = get_user(config->glewlwyd_config, username, NULL);
   int ret;
+  const char *passwords[1];
   
+  passwords[0] = password;
   if (check_result_value(j_user, G_OK)) {
-    ret = user_set_password(config->glewlwyd_config, username, password);
+    ret = user_set_password(config->glewlwyd_config, username, passwords, 1);
   } else if (check_result_value(j_user, G_ERROR_NOT_FOUND)) {
     ret = U_ERROR_NOT_FOUND;
   } else {
@@ -544,6 +546,14 @@ int glewlwyd_plugin_callback_delete_client(struct config_plugin * config, const
   return delete_client(config->glewlwyd_config, client_id, NULL);
 }
 
+json_t * glewlwyd_plugin_callback_get_scheme_list(struct config_plugin * config, const char * username) {
+  if (o_strlen(username)) {
+    return get_scheme_list_for_user(config->glewlwyd_config, username);
+  } else {
+    return json_pack("{si}", "result", G_ERROR_PARAM);
+  }
+}
+
 json_t * glewlwyd_plugin_callback_scheme_register(struct config_plugin * config, const char * mod_name, const struct _u_request * http_request, const char * username, json_t * j_scheme_data) {
   json_t * j_return;
   struct _user_auth_scheme_module_instance * scheme_instance = get_user_auth_scheme_module_instance(config->glewlwyd_config, mod_name);
