@@ -8,7 +8,7 @@
  * 
  * user management functions definition
  *
- * Copyright 2016-2020 Nicolas Mora <mail@babelouest.org>
+ * Copyright 2016-2021 Nicolas Mora <mail@babelouest.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU GENERAL PUBLIC LICENSE
@@ -74,31 +74,66 @@ json_t * auth_check_user_scheme(struct config_elements * config, const char * sc
   json_t * j_return = NULL, * j_user;
   int res;
   
-  j_user = get_user(config, username, NULL);
-  if (check_result_value(j_user, G_OK) && json_object_get(json_object_get(j_user, "user"), "enabled") == json_true()) {
-    scheme_instance = get_user_auth_scheme_module_instance(config, scheme_name);
-    if (scheme_instance != NULL && 0 == o_strcmp(scheme_type, scheme_instance->module->name) && scheme_instance->enabled) {
-      res = scheme_instance->module->user_auth_scheme_module_validate(config->config_m, request, username, j_scheme_value, scheme_instance->cls);
-      if (res == G_OK || res == G_ERROR_UNAUTHORIZED || res == G_ERROR_PARAM || res == G_ERROR_NOT_FOUND || res == G_ERROR) {
-        j_return = json_pack("{si}", "result", res);
+  if ((res = user_has_scheme(config, username, scheme_name)) == G_OK) {
+    j_user = get_user(config, username, NULL);
+    if (check_result_value(j_user, G_OK) && json_object_get(json_object_get(j_user, "user"), "enabled") == json_true()) {
+      scheme_instance = get_user_auth_scheme_module_instance(config, scheme_name);
+      if (scheme_instance != NULL && 0 == o_strcmp(scheme_type, scheme_instance->module->name) && scheme_instance->enabled) {
+        res = scheme_instance->module->user_auth_scheme_module_validate(config->config_m, request, username, j_scheme_value, scheme_instance->cls);
+        if (res == G_OK || res == G_ERROR_UNAUTHORIZED || res == G_ERROR_PARAM || res == G_ERROR_NOT_FOUND || res == G_ERROR) {
+          j_return = json_pack("{si}", "result", res);
+        } else {
+          y_log_message(Y_LOG_LEVEL_ERROR, "auth_check_user_scheme - Error unrecognize return value for user_auth_scheme_module_validate: %d", res);
+          j_return = json_pack("{si}", "result", G_ERROR);
+        }
       } else {
-        y_log_message(Y_LOG_LEVEL_ERROR, "auth_check_user_scheme - Error unrecognize return value for user_auth_scheme_module_validate: %d", res);
-        j_return = json_pack("{si}", "result", G_ERROR);
+        j_return = json_pack("{si}", "result", G_ERROR_UNAUTHORIZED);
       }
     } else {
       j_return = json_pack("{si}", "result", G_ERROR_UNAUTHORIZED);
     }
+    json_decref(j_user);
+  } else if (res != G_ERROR_NOT_FOUND) {
+    y_log_message(Y_LOG_LEVEL_ERROR, "auth_check_user_scheme - Error user_has_scheme");
+    j_return = json_pack("{si}", "result", G_ERROR);
+  } else {
+    j_return = json_pack("{si}", "result", G_ERROR_UNAUTHORIZED);
+  }
+  return j_return;
+}
+
+json_t * auth_check_identify_scheme(struct config_elements * config, const char * scheme_type, const char * scheme_name, json_t * j_scheme_value, const struct _u_request * request) {
+  struct _user_auth_scheme_module_instance * scheme_instance;
+  json_t * j_return = NULL, * j_user, * j_response;
+  
+  scheme_instance = get_user_auth_scheme_module_instance(config, scheme_name);
+  if (scheme_instance != NULL && 0 == o_strcmp(scheme_type, scheme_instance->module->name) && scheme_instance->enabled) {
+    j_response = scheme_instance->module->user_auth_scheme_module_identify(config->config_m, request, j_scheme_value, scheme_instance->cls);
+    if (check_result_value(j_response, G_OK)) {
+      j_user = get_user(config, json_string_value(json_object_get(j_response, "username")), NULL);
+      if (check_result_value(j_user, G_OK) && json_object_get(json_object_get(j_user, "user"), "enabled") == json_true()) {
+        j_return = json_pack("{sisO}", "result", G_OK, "username", json_object_get(j_response, "username"));
+      } else {
+        j_return = json_pack("{si}", "result", G_ERROR_UNAUTHORIZED);
+      }
+      json_decref(j_user);
+    } else if (!check_result_value(j_response, G_ERROR)) {
+      j_return = json_incref(j_response);
+    } else {
+      y_log_message(Y_LOG_LEVEL_ERROR, "auth_check_identify_scheme - Error user_auth_scheme_module_identify");
+      j_return = json_pack("{si}", "result", G_ERROR);
+    }
+    json_decref(j_response);
   } else {
     j_return = json_pack("{si}", "result", G_ERROR_UNAUTHORIZED);
   }
-  json_decref(j_user);
   return j_return;
 }
 
 json_t * auth_trigger_user_scheme(struct config_elements * config, const char * scheme_type, const char * scheme_name, const char * username, json_t * j_trigger_parameters, const struct _u_request * request) {
   struct _user_auth_scheme_module_instance * scheme_instance;
   json_t * j_return = NULL, * j_response = NULL;
-  
+
   scheme_instance = get_user_auth_scheme_module_instance(config, scheme_name);
   if (scheme_instance != NULL && 0 == o_strcmp(scheme_type, scheme_instance->module->name) && scheme_instance->enabled) {
     j_response = scheme_instance->module->user_auth_scheme_module_trigger(config->config_m, request, username, j_trigger_parameters, scheme_instance->cls);
@@ -121,40 +156,74 @@ json_t * auth_trigger_user_scheme(struct config_elements * config, const char *
   return j_return;
 }
 
+json_t * auth_trigger_identify_scheme(struct config_elements * config, const char * scheme_type, const char * scheme_name, json_t * j_trigger_parameters, const struct _u_request * request) {
+  struct _user_auth_scheme_module_instance * scheme_instance;
+  json_t * j_return = NULL, * j_response = NULL;
+
+  scheme_instance = get_user_auth_scheme_module_instance(config, scheme_name);
+  if (scheme_instance != NULL && 0 == o_strcmp(scheme_type, scheme_instance->module->name) && scheme_instance->enabled) {
+    j_response = scheme_instance->module->user_auth_scheme_module_identify(config->config_m, request, j_trigger_parameters, scheme_instance->cls);
+    if (check_result_value(j_response, G_OK)) {
+      if (json_object_get(j_response, "response") != NULL) {
+        j_return = json_pack("{sisO}", "result", G_OK, "trigger", json_object_get(j_response, "response"));
+      } else {
+        j_return = json_pack("{si}", "result", G_OK);
+      }
+    } else if (!check_result_value(j_response, G_ERROR)) {
+      j_return = json_incref(j_response);
+    } else {
+      y_log_message(Y_LOG_LEVEL_ERROR, "auth_trigger_identify_scheme - Error user_auth_scheme_module_trigger");
+      j_return = json_pack("{si}", "result", G_ERROR);
+    }
+    json_decref(j_response);
+  } else {
+    j_return = json_pack("{si}", "result", G_ERROR_UNAUTHORIZED);
+  }
+  return j_return;
+}
+
 json_t * auth_register_user_scheme(struct config_elements * config, const char * scheme_type, const char * scheme_name, const char * username, int delegate, json_t * j_register_parameters, const struct _u_request * request) {
   struct _user_auth_scheme_module_instance * scheme_instance;
   json_t * j_return = NULL, * j_response = NULL;
+  int res;
   
-  if (json_is_object(j_register_parameters)) {
-    scheme_instance = get_user_auth_scheme_module_instance(config, scheme_name);
-    if (scheme_instance != NULL && 0 == o_strcmp(scheme_type, scheme_instance->module->name) && scheme_instance->enabled) {
-      if (delegate || scheme_instance->guasmi_allow_user_register) {
-        j_response = scheme_instance->module->user_auth_scheme_module_register(config->config_m, request, username, j_register_parameters, scheme_instance->cls);
-        if (check_result_value(j_response, G_OK)) {
-          if (json_object_get(j_response, "response") != NULL) {
-            j_return = json_pack("{sisO}", "result", G_OK, "register", json_object_get(j_response, "response"));
+  if ((res = user_has_scheme(config, username, scheme_name)) == G_OK) {
+    if (json_is_object(j_register_parameters)) {
+      scheme_instance = get_user_auth_scheme_module_instance(config, scheme_name);
+      if (scheme_instance != NULL && 0 == o_strcmp(scheme_type, scheme_instance->module->name) && scheme_instance->enabled) {
+        if (delegate || scheme_instance->guasmi_allow_user_register) {
+          j_response = scheme_instance->module->user_auth_scheme_module_register(config->config_m, request, username, j_register_parameters, scheme_instance->cls);
+          if (check_result_value(j_response, G_OK)) {
+            if (json_object_get(j_response, "response") != NULL) {
+              j_return = json_pack("{sisO}", "result", G_OK, "register", json_object_get(j_response, "response"));
+            } else {
+              j_return = json_pack("{si}", "result", G_OK);
+            }
+          } else if (j_response != NULL && !check_result_value(j_response, G_ERROR)) {
+            if (json_object_get(j_response, "response") != NULL) {
+              j_return = json_pack("{sIsO}", "result", json_integer_value(json_object_get(j_response, "result")), "register", json_object_get(j_response, "response"));
+            } else {
+              j_return = json_pack("{sI}", "result", json_integer_value(json_object_get(j_response, "result")));
+            }
           } else {
-            j_return = json_pack("{si}", "result", G_OK);
-          }
-        } else if (j_response != NULL && !check_result_value(j_response, G_ERROR)) {
-          if (json_object_get(j_response, "response") != NULL) {
-            j_return = json_pack("{sIsO}", "result", json_integer_value(json_object_get(j_response, "result")), "register", json_object_get(j_response, "response"));
-          } else {
-            j_return = json_pack("{sI}", "result", json_integer_value(json_object_get(j_response, "result")));
+            y_log_message(Y_LOG_LEVEL_ERROR, "auth_register_user_scheme - Error user_auth_scheme_module_register");
+            j_return = json_pack("{si}", "result", G_ERROR);
           }
+          json_decref(j_response);
         } else {
-          y_log_message(Y_LOG_LEVEL_ERROR, "auth_register_user_scheme - Error user_auth_scheme_module_register");
-          j_return = json_pack("{si}", "result", G_ERROR);
+          j_return = json_pack("{si}", "result", G_ERROR_UNAUTHORIZED);
         }
-        json_decref(j_response);
       } else {
         j_return = json_pack("{si}", "result", G_ERROR_UNAUTHORIZED);
       }
     } else {
-      j_return = json_pack("{si}", "result", G_ERROR_UNAUTHORIZED);
+      j_return = json_pack("{si}", "result", G_ERROR_PARAM);
     }
+  } else if (res != G_ERROR_NOT_FOUND) {
+    y_log_message(Y_LOG_LEVEL_ERROR, "auth_register_user_scheme - Error user_has_scheme");
+    j_return = json_pack("{si}", "result", G_ERROR);
   } else {
-    j_return = json_pack("{si}", "result", G_ERROR_PARAM);
+    j_return = json_pack("{si}", "result", G_ERROR_UNAUTHORIZED);
   }
   return j_return;
 }
@@ -162,23 +231,31 @@ json_t * auth_register_user_scheme(struct config_elements * config, const char *
 json_t * auth_register_get_user_scheme(struct config_elements * config, const char * scheme_type, const char * scheme_name, const char * username, const struct _u_request * request) {
   struct _user_auth_scheme_module_instance * scheme_instance;
   json_t * j_return = NULL, * j_response = NULL;
+  int res;
   
-  scheme_instance = get_user_auth_scheme_module_instance(config, scheme_name);
-  if (scheme_instance != NULL && 0 == o_strcmp(scheme_type, scheme_instance->module->name) && scheme_instance->enabled) {
-    j_response = scheme_instance->module->user_auth_scheme_module_register_get(config->config_m, request, username, scheme_instance->cls);
-    if (check_result_value(j_response, G_OK)) {
-      if (json_object_get(j_response, "response") != NULL) {
-        j_return = json_pack("{sisO}", "result", G_OK, "register", json_object_get(j_response, "response"));
+  if ((res = user_has_scheme(config, username, scheme_name)) == G_OK) {
+    scheme_instance = get_user_auth_scheme_module_instance(config, scheme_name);
+    if (scheme_instance != NULL && 0 == o_strcmp(scheme_type, scheme_instance->module->name) && scheme_instance->enabled) {
+      j_response = scheme_instance->module->user_auth_scheme_module_register_get(config->config_m, request, username, scheme_instance->cls);
+      if (check_result_value(j_response, G_OK)) {
+        if (json_object_get(j_response, "response") != NULL) {
+          j_return = json_pack("{sisO}", "result", G_OK, "register", json_object_get(j_response, "response"));
+        } else {
+          j_return = json_pack("{si}", "result", G_OK);
+        }
+      } else if (!check_result_value(j_response, G_ERROR)) {
+        j_return = json_incref(j_response);
       } else {
-        j_return = json_pack("{si}", "result", G_OK);
+        y_log_message(Y_LOG_LEVEL_ERROR, "auth_register_get_user_scheme - Error user_auth_scheme_module_register_get");
+        j_return = json_pack("{si}", "result", G_ERROR);
       }
-    } else if (!check_result_value(j_response, G_ERROR)) {
-      j_return = json_incref(j_response);
+      json_decref(j_response);
     } else {
-      y_log_message(Y_LOG_LEVEL_ERROR, "auth_register_get_user_scheme - Error user_auth_scheme_module_register_get");
-      j_return = json_pack("{si}", "result", G_ERROR);
+      j_return = json_pack("{si}", "result", G_ERROR_UNAUTHORIZED);
     }
-    json_decref(j_response);
+  } else if (res != G_ERROR_NOT_FOUND) {
+    y_log_message(Y_LOG_LEVEL_ERROR, "auth_register_get_user_scheme - Error user_has_scheme");
+    j_return = json_pack("{si}", "result", G_ERROR);
   } else {
     j_return = json_pack("{si}", "result", G_ERROR_UNAUTHORIZED);
   }
@@ -197,6 +274,37 @@ int user_has_scope(json_t * j_user, const char * scope) {
   return 0;
 }
 
+int user_has_scheme(struct config_elements * config, const char * username, const char * scheme_name) {
+  json_t * j_user, * j_element = NULL, * j_group = NULL, * j_scheme = NULL, * j_scope = NULL;
+  size_t index = 0, index_s = 0;
+  const char * group = NULL;
+  int ret = G_ERROR_NOT_FOUND;
+  
+  j_user = get_user(config, username, NULL);
+  if (check_result_value(j_user, G_OK)) {
+    json_array_foreach(json_object_get(json_object_get(j_user, "user"), "scope"), index, j_element) {
+      j_scope = get_scope(config, json_string_value(j_element));
+      if (check_result_value(j_scope, G_OK)) {
+        json_object_foreach(json_object_get(json_object_get(j_scope, "scope"), "scheme"), group, j_group) {
+          json_array_foreach(j_group, index_s, j_scheme) {
+            if (0 == o_strcmp(json_string_value(json_object_get(j_scheme, "scheme_name")), scheme_name)) {
+              ret = G_OK;
+            }
+          }
+        }
+      } else {
+        y_log_message(Y_LOG_LEVEL_ERROR, "user_has_scheme - Error get_scope '%s'", json_string_value(j_element));
+      }
+      json_decref(j_scope);
+    }
+  } else if (!check_result_value(j_user, G_ERROR_NOT_FOUND)) {
+    y_log_message(Y_LOG_LEVEL_ERROR, "user_has_scheme - Error get_user");
+    ret = G_ERROR;
+  }
+  json_decref(j_user);
+  return ret;
+}
+
 json_t * get_user(struct config_elements * config, const char * username, const char * source) {
   int found = 0;
   json_t * j_return = NULL, * j_user, * j_module_list, * j_module;
@@ -344,30 +452,32 @@ json_t * get_user_list(struct config_elements * config, const char * pattern, si
       j_return = json_pack("{sis[]}", "result", G_OK, "user");
       if (j_return != NULL) {
         json_array_foreach(json_object_get(j_module_list, "module"), index, j_module) {
-          user_module = get_user_module_instance(config, json_string_value(json_object_get(j_module, "name")));
-          if (user_module != NULL && user_module->enabled) {
-            if ((count_total = user_module->module->user_module_count_total(config->config_m, pattern, user_module->cls)) > cur_offset && cur_limit) {
-              j_result = user_module->module->user_module_get_list(config->config_m, pattern, cur_offset, cur_limit, user_module->cls);
-              if (check_result_value(j_result, G_OK)) {
-                json_array_foreach(json_object_get(j_result, "list"), index_u, j_element) {
-                  json_object_set_new(j_element, "source", json_string(user_module->name));
-                }
-                cur_offset = 0;
-                if (cur_limit > json_array_size(json_object_get(j_result, "list"))) {
-                  cur_limit -= json_array_size(json_object_get(j_result, "list"));
+          if (cur_limit) {
+            user_module = get_user_module_instance(config, json_string_value(json_object_get(j_module, "name")));
+            if (user_module != NULL && user_module->enabled) {
+              if ((count_total = user_module->module->user_module_count_total(config->config_m, pattern, user_module->cls)) > cur_offset && cur_limit) {
+                j_result = user_module->module->user_module_get_list(config->config_m, pattern, cur_offset, cur_limit, user_module->cls);
+                if (check_result_value(j_result, G_OK)) {
+                  json_array_foreach(json_object_get(j_result, "list"), index_u, j_element) {
+                    json_object_set_new(j_element, "source", json_string(user_module->name));
+                  }
+                  cur_offset = 0;
+                  if (cur_limit > json_array_size(json_object_get(j_result, "list"))) {
+                    cur_limit -= json_array_size(json_object_get(j_result, "list"));
+                  } else {
+                    cur_limit = 0;
+                  }
+                  json_array_extend(json_object_get(j_return, "user"), json_object_get(j_result, "list"));
                 } else {
-                  cur_limit = 0;
+                  y_log_message(Y_LOG_LEVEL_ERROR, "get_user_list - Error user_module_get_list for module %s", json_string_value(json_object_get(j_module, "name")));
                 }
-                json_array_extend(json_object_get(j_return, "user"), json_object_get(j_result, "list"));
+                json_decref(j_result);
               } else {
-                y_log_message(Y_LOG_LEVEL_ERROR, "get_user_list - Error user_module_get_list for module %s", json_string_value(json_object_get(j_module, "name")));
+                cur_offset -= count_total;
               }
-              json_decref(j_result);
-            } else {
-              cur_offset -= count_total;
+            } else if (user_module == NULL) {
+              y_log_message(Y_LOG_LEVEL_ERROR, "get_user_list - Error, user_module_instance %s is NULL", json_string_value(json_object_get(j_module, "name")));
             }
-          } else if (user_module == NULL) {
-            y_log_message(Y_LOG_LEVEL_ERROR, "get_user_list - Error, user_module_instance %s is NULL", json_string_value(json_object_get(j_module, "name")));
           }
         }
       } else {
@@ -685,7 +795,7 @@ int user_delete_profile(struct config_elements * config, const char * username)
   return ret;
 }
 
-int user_update_password(struct config_elements * config, const char * username, const char * old_password, const char * new_password) {
+int user_update_password(struct config_elements * config, const char * username, const char * old_password, const char ** new_passwords, size_t new_passwords_len) {
   json_t * j_user = get_user(config, username, NULL);
   struct _user_module_instance * user_module;
   int ret;
@@ -694,7 +804,7 @@ int user_update_password(struct config_elements * config, const char * username,
     user_module = get_user_module_instance(config, json_string_value(json_object_get(json_object_get(j_user, "user"), "source")));
     if (user_module != NULL && user_module->enabled && !user_module->readonly) {
       if ((ret = user_module->module->user_module_check_password(config->config_m, username, old_password, user_module->cls)) == G_OK) {
-        ret = user_module->module->user_module_update_password(config->config_m, username, new_password, user_module->cls);
+        ret = user_module->module->user_module_update_password(config->config_m, username, new_passwords, new_passwords_len, user_module->cls);
       } else if (ret == G_ERROR_UNAUTHORIZED) {
         ret = G_ERROR_PARAM;
       } else {
@@ -717,7 +827,7 @@ int user_update_password(struct config_elements * config, const char * username,
   return ret;
 }
 
-int user_set_password(struct config_elements * config, const char * username, const char * new_password) {
+int user_set_password(struct config_elements * config, const char * username, const char ** new_passwords, size_t new_passwords_len) {
   json_t * j_user = get_user(config, username, NULL);
   struct _user_module_instance * user_module;
   int ret;
@@ -725,7 +835,7 @@ int user_set_password(struct config_elements * config, const char * username, co
   if (check_result_value(j_user, G_OK)) {
     user_module = get_user_module_instance(config, json_string_value(json_object_get(json_object_get(j_user, "user"), "source")));
     if (user_module != NULL && user_module->enabled && !user_module->readonly) {
-      ret = user_module->module->user_module_update_password(config->config_m, username, new_password, user_module->cls);
+      ret = user_module->module->user_module_update_password(config->config_m, username, new_passwords, new_passwords_len, user_module->cls);
     } else if (user_module != NULL && (user_module->readonly || !user_module->enabled)) {
       ret = G_ERROR_PARAM;
     } else {
