@@ -1,6 +1,7 @@
 DROP TABLE IF EXISTS g_user_property;
 DROP TABLE IF EXISTS g_user_scope_user;
 DROP TABLE IF EXISTS g_user_scope;
+DROP TABLE IF EXISTS g_user_password;
 DROP TABLE IF EXISTS g_user;
 
 CREATE TABLE g_user (
@@ -8,7 +9,6 @@ CREATE TABLE g_user (
   gu_username VARCHAR(128) NOT NULL UNIQUE,
   gu_name VARCHAR(256) DEFAULT '',
   gu_email VARCHAR(512) DEFAULT '',
-  gu_password VARCHAR(256),
   gu_enabled TINYINT(1) DEFAULT 1
 );
 
@@ -35,3 +35,10 @@ CREATE TABLE g_user_property (
   FOREIGN KEY(gu_id) REFERENCES g_user(gu_id) ON DELETE CASCADE
 );
 CREATE INDEX i_g_user_property_name ON g_user_property(gup_name);
+
+CREATE TABLE g_user_password (
+  guw_id INT(11) PRIMARY KEY AUTO_INCREMENT,
+  gu_id INT(11),
+  guw_password VARCHAR(256),
+  FOREIGN KEY(gu_id) REFERENCES g_user(gu_id) ON DELETE CASCADE
+);
