@@ -8,6 +8,7 @@
 
 #include <check.h>
 #include <ulfius.h>
+#include <rhonabwy.h>
 #include <orcania.h>
 #include <yder.h>
 
@@ -99,6 +100,7 @@ START_TEST(test_oidc_code_ok)
   char ** id_token_split = NULL, * str_payload = NULL, at_hash[33], at_hash_encoded[64];
   size_t str_payload_len = 0, at_hash_len = 33, at_hash_encoded_len = 0;
   gnutls_datum_t at_data;
+  jwt_t * jwt;
   
   ulfius_init_request(&req);
   ulfius_init_response(&resp);
@@ -116,6 +118,10 @@ START_TEST(test_oidc_code_ok)
   ck_assert_ptr_ne(json_object_get(j_body, "access_token"), NULL);
   ck_assert_ptr_ne(json_object_get(j_body, "id_token"), NULL);
   
+  ck_assert_int_eq(r_jwt_init(&jwt), RHN_OK);
+  ck_assert_int_eq(r_jwt_parse(jwt, json_string_value(json_object_get(j_body, "access_token")), 0), RHN_OK);
+  r_jwt_free(jwt);
+  
   ck_assert_int_eq(split_string(json_string_value(json_object_get(j_body, "id_token")), ".", &id_token_split), 3);
   ck_assert_int_eq(o_base64url_decode((unsigned char *)id_token_split[1], o_strlen(id_token_split[1]), NULL, &str_payload_len), 1);
   ck_assert_ptr_ne((str_payload = o_malloc(str_payload_len + 1)), NULL);
@@ -422,6 +428,7 @@ START_TEST(test_oidc_code_retry_with_max_use)
   struct _u_response auth_resp, code_resp;
   json_t * j_body;
   char * cookie, * code;
+  jwt_t * jwt;
 
   ulfius_init_request(&auth_req);
   ulfius_init_response(&auth_resp);
@@ -581,6 +588,12 @@ START_TEST(test_oidc_code_retry_with_max_use)
   ck_assert_int_eq(code_resp.status, 200);
   j_body = ulfius_get_json_body_response(&code_resp, NULL);
   ck_assert_ptr_ne(j_body, NULL);
+  
+  ck_assert_int_eq(r_jwt_init(&jwt), RHN_OK);
+  ck_assert_int_eq(r_jwt_parse(jwt, json_string_value(json_object_get(j_body, "access_token")), 0), RHN_OK);
+  ck_assert_str_eq(SCOPE_LIST_MAX_USE, r_jwt_get_claim_str_value(jwt, "aud"));
+  r_jwt_free(jwt);
+  
   ck_assert_str_eq(json_string_value(json_object_get(j_body, "scope")), SCOPE_LIST_MAX_USE);
   ulfius_clean_request(&code_req);
   ulfius_clean_response(&code_resp);
