@@ -37,6 +37,7 @@
 #define CLIENT_ID "client_device"
 #define CLIENT_NAME "client for device"
 #define CLIENT_SECRET "very-secret"
+#define RESOURCE "https://resource.tld/"
 
 struct _u_request admin_req;
 struct _u_request user_req;
@@ -145,7 +146,7 @@ START_TEST(test_oidc_device_authorization_client_cred_post_valid)
   ck_assert_ptr_ne(j_resp = ulfius_get_json_body_response(&resp, NULL), NULL);
   ck_assert_ptr_ne(json_object_get(j_resp, "device_code"), NULL);
   ck_assert_ptr_ne(json_object_get(j_resp, "user_code"), NULL);
-  ck_assert_str_eq(json_string_value(json_object_get(j_resp, "verification_uri")), "http://localhost:4593/api/oidc_device/device");
+  ck_assert_str_eq(json_string_value(json_object_get(j_resp, "verification_uri")), "http://localhost:4593/api/" PLUGIN_NAME "/device");
   ck_assert_ptr_ne(json_object_get(j_resp, "verification_uri_complete"), NULL);
   ck_assert_int_eq(json_integer_value(json_object_get(j_resp, "expires_in")), 600);
   ck_assert_int_eq(json_integer_value(json_object_get(j_resp, "interval")), 5);
@@ -177,7 +178,7 @@ START_TEST(test_oidc_device_authorization_client_cred_header_valid)
   ck_assert_ptr_ne(j_resp = ulfius_get_json_body_response(&resp, NULL), NULL);
   ck_assert_ptr_ne(json_object_get(j_resp, "device_code"), NULL);
   ck_assert_ptr_ne(json_object_get(j_resp, "user_code"), NULL);
-  ck_assert_str_eq(json_string_value(json_object_get(j_resp, "verification_uri")), "http://localhost:4593/api/oidc_device/device");
+  ck_assert_str_eq(json_string_value(json_object_get(j_resp, "verification_uri")), "http://localhost:4593/api/" PLUGIN_NAME "/device");
   ck_assert_ptr_ne(json_object_get(j_resp, "verification_uri_complete"), NULL);
   ck_assert_int_eq(json_integer_value(json_object_get(j_resp, "expires_in")), 600);
   ck_assert_int_eq(json_integer_value(json_object_get(j_resp, "interval")), 5);
@@ -231,7 +232,7 @@ START_TEST(test_oidc_device_authorization_client_cred_header_no_post_valid)
   ck_assert_ptr_ne(j_resp = ulfius_get_json_body_response(&resp, NULL), NULL);
   ck_assert_ptr_ne(json_object_get(j_resp, "device_code"), NULL);
   ck_assert_ptr_ne(json_object_get(j_resp, "user_code"), NULL);
-  ck_assert_str_eq(json_string_value(json_object_get(j_resp, "verification_uri")), "http://localhost:4593/api/oidc_device/device");
+  ck_assert_str_eq(json_string_value(json_object_get(j_resp, "verification_uri")), "http://localhost:4593/api/" PLUGIN_NAME "/device");
   ck_assert_ptr_ne(json_object_get(j_resp, "verification_uri_complete"), NULL);
   ck_assert_int_eq(json_integer_value(json_object_get(j_resp, "expires_in")), 600);
   ck_assert_int_eq(json_integer_value(json_object_get(j_resp, "interval")), 5);
@@ -261,7 +262,7 @@ START_TEST(test_oidc_device_authorization_client_pub_post_valid)
   ck_assert_ptr_ne(j_resp = ulfius_get_json_body_response(&resp, NULL), NULL);
   ck_assert_ptr_ne(json_object_get(j_resp, "device_code"), NULL);
   ck_assert_ptr_ne(json_object_get(j_resp, "user_code"), NULL);
-  ck_assert_str_eq(json_string_value(json_object_get(j_resp, "verification_uri")), "http://localhost:4593/api/oidc_device/device");
+  ck_assert_str_eq(json_string_value(json_object_get(j_resp, "verification_uri")), "http://localhost:4593/api/" PLUGIN_NAME "/device");
   ck_assert_ptr_ne(json_object_get(j_resp, "verification_uri_complete"), NULL);
   ck_assert_int_eq(json_integer_value(json_object_get(j_resp, "expires_in")), 600);
   ck_assert_int_eq(json_integer_value(json_object_get(j_resp, "interval")), 5);
@@ -315,7 +316,7 @@ START_TEST(test_oidc_device_authorization_user_code_redirect_login_valid)
   ck_assert_ptr_ne(j_resp = ulfius_get_json_body_response(&resp, NULL), NULL);
   ck_assert_ptr_ne(json_object_get(j_resp, "device_code"), NULL);
   ck_assert_ptr_ne(json_object_get(j_resp, "user_code"), NULL);
-  ck_assert_str_eq(json_string_value(json_object_get(j_resp, "verification_uri")), "http://localhost:4593/api/oidc_device/device");
+  ck_assert_str_eq(json_string_value(json_object_get(j_resp, "verification_uri")), "http://localhost:4593/api/" PLUGIN_NAME "/device");
   ck_assert_ptr_ne(json_object_get(j_resp, "verification_uri_complete"), NULL);
   ck_assert_int_eq(json_integer_value(json_object_get(j_resp, "expires_in")), 600);
   ck_assert_int_eq(json_integer_value(json_object_get(j_resp, "interval")), 5);
@@ -358,7 +359,7 @@ START_TEST(test_oidc_device_authorization_user_code_input_invalid)
   ck_assert_ptr_ne(j_resp = ulfius_get_json_body_response(&resp, NULL), NULL);
   ck_assert_ptr_ne(json_object_get(j_resp, "device_code"), NULL);
   ck_assert_ptr_ne(json_object_get(j_resp, "user_code"), NULL);
-  ck_assert_str_eq(json_string_value(json_object_get(j_resp, "verification_uri")), "http://localhost:4593/api/oidc_device/device");
+  ck_assert_str_eq(json_string_value(json_object_get(j_resp, "verification_uri")), "http://localhost:4593/api/" PLUGIN_NAME "/device");
   ck_assert_ptr_ne(json_object_get(j_resp, "verification_uri_complete"), NULL);
   ck_assert_int_eq(json_integer_value(json_object_get(j_resp, "expires_in")), 600);
   ck_assert_int_eq(json_integer_value(json_object_get(j_resp, "interval")), 5);
@@ -402,7 +403,7 @@ START_TEST(test_oidc_device_authorization_user_code_input_redirect_login_valid)
   ck_assert_ptr_ne(json_object_get(j_resp, "device_code"), NULL);
   ck_assert_ptr_ne(json_object_get(j_resp, "user_code"), NULL);
   ck_assert_ptr_ne(code = json_string_value(json_object_get(j_resp, "user_code")), NULL);
-  ck_assert_str_eq(json_string_value(json_object_get(j_resp, "verification_uri")), "http://localhost:4593/api/oidc_device/device");
+  ck_assert_str_eq(json_string_value(json_object_get(j_resp, "verification_uri")), "http://localhost:4593/api/" PLUGIN_NAME "/device");
   ck_assert_ptr_ne(json_object_get(j_resp, "verification_uri_complete"), NULL);
   ck_assert_int_eq(json_integer_value(json_object_get(j_resp, "expires_in")), 600);
   ck_assert_int_eq(json_integer_value(json_object_get(j_resp, "interval")), 5);
@@ -446,7 +447,7 @@ START_TEST(test_oidc_device_authorization_user_code_input_complete_valid)
   ck_assert_ptr_ne(json_object_get(j_resp, "device_code"), NULL);
   ck_assert_ptr_ne(json_object_get(j_resp, "user_code"), NULL);
   ck_assert_ptr_ne(code = json_string_value(json_object_get(j_resp, "user_code")), NULL);
-  ck_assert_str_eq(json_string_value(json_object_get(j_resp, "verification_uri")), "http://localhost:4593/api/oidc_device/device");
+  ck_assert_str_eq(json_string_value(json_object_get(j_resp, "verification_uri")), "http://localhost:4593/api/" PLUGIN_NAME "/device");
   ck_assert_ptr_ne(json_object_get(j_resp, "verification_uri_complete"), NULL);
   ck_assert_int_eq(json_integer_value(json_object_get(j_resp, "expires_in")), 600);
   ck_assert_int_eq(json_integer_value(json_object_get(j_resp, "interval")), 5);
@@ -483,6 +484,7 @@ START_TEST(test_oidc_device_authorization_device_verification_valid)
   struct _u_response resp;
   json_t * j_resp, * j_grant;
   const char * redirect_uri, * code, * device_code;
+  jwt_t * jwt;
   
   ck_assert_int_eq(ulfius_init_request(&req), U_OK);
   ck_assert_int_eq(ulfius_init_response(&resp), U_OK);
@@ -501,7 +503,7 @@ START_TEST(test_oidc_device_authorization_device_verification_valid)
   ck_assert_ptr_ne(json_object_get(j_resp, "user_code"), NULL);
   ck_assert_ptr_ne(code = json_string_value(json_object_get(j_resp, "user_code")), NULL);
   ck_assert_ptr_ne(device_code = json_string_value(json_object_get(j_resp, "device_code")), NULL);
-  ck_assert_str_eq(json_string_value(json_object_get(j_resp, "verification_uri")), "http://localhost:4593/api/oidc_device/device");
+  ck_assert_str_eq(json_string_value(json_object_get(j_resp, "verification_uri")), "http://localhost:4593/api/" PLUGIN_NAME "/device");
   ck_assert_ptr_ne(json_object_get(j_resp, "verification_uri_complete"), NULL);
   ck_assert_int_eq(json_integer_value(json_object_get(j_resp, "expires_in")), 600);
   ck_assert_int_eq(json_integer_value(json_object_get(j_resp, "interval")), 5);
@@ -546,6 +548,11 @@ START_TEST(test_oidc_device_authorization_device_verification_valid)
   ck_assert_ptr_ne(json_object_get(j_resp, "refresh_token"), NULL);
   ck_assert_ptr_ne(json_object_get(j_resp, "id_token"), NULL);
   
+  ck_assert_int_eq(r_jwt_init(&jwt), RHN_OK);
+  ck_assert_int_eq(r_jwt_parse(jwt, json_string_value(json_object_get(j_resp, "access_token")), 0), RHN_OK);
+  ck_assert_str_eq(SCOPE_LIST, r_jwt_get_claim_str_value(jwt, "aud"));
+  r_jwt_free(jwt);
+  
   json_decref(j_resp);
   ulfius_clean_request(&req);
   ulfius_clean_response(&resp);
@@ -577,7 +584,7 @@ START_TEST(test_oidc_device_authorization_device_verification_auth_pending)
   ck_assert_ptr_ne(json_object_get(j_resp, "user_code"), NULL);
   ck_assert_ptr_ne(code = json_string_value(json_object_get(j_resp, "user_code")), NULL);
   ck_assert_ptr_ne(device_code = json_string_value(json_object_get(j_resp, "device_code")), NULL);
-  ck_assert_str_eq(json_string_value(json_object_get(j_resp, "verification_uri")), "http://localhost:4593/api/oidc_device/device");
+  ck_assert_str_eq(json_string_value(json_object_get(j_resp, "verification_uri")), "http://localhost:4593/api/" PLUGIN_NAME "/device");
   ck_assert_ptr_ne(json_object_get(j_resp, "verification_uri_complete"), NULL);
   ck_assert_int_eq(json_integer_value(json_object_get(j_resp, "expires_in")), 600);
   ck_assert_int_eq(json_integer_value(json_object_get(j_resp, "interval")), 5);
@@ -632,7 +639,7 @@ START_TEST(test_oidc_device_authorization_device_verification_auth_slow_down)
   ck_assert_ptr_ne(json_object_get(j_resp, "user_code"), NULL);
   ck_assert_ptr_ne(code = json_string_value(json_object_get(j_resp, "user_code")), NULL);
   ck_assert_ptr_ne(device_code = json_string_value(json_object_get(j_resp, "device_code")), NULL);
-  ck_assert_str_eq(json_string_value(json_object_get(j_resp, "verification_uri")), "http://localhost:4593/api/oidc_device/device");
+  ck_assert_str_eq(json_string_value(json_object_get(j_resp, "verification_uri")), "http://localhost:4593/api/" PLUGIN_NAME "/device");
   ck_assert_ptr_ne(json_object_get(j_resp, "verification_uri_complete"), NULL);
   ck_assert_int_eq(json_integer_value(json_object_get(j_resp, "expires_in")), 600);
   ck_assert_int_eq(json_integer_value(json_object_get(j_resp, "interval")), 5);
@@ -694,7 +701,7 @@ START_TEST(test_oidc_device_authorization_device_verification_device_code_invali
   ck_assert_ptr_ne(json_object_get(j_resp, "device_code"), NULL);
   ck_assert_ptr_ne(json_object_get(j_resp, "user_code"), NULL);
   ck_assert_ptr_ne(code = json_string_value(json_object_get(j_resp, "user_code")), NULL);
-  ck_assert_str_eq(json_string_value(json_object_get(j_resp, "verification_uri")), "http://localhost:4593/api/oidc_device/device");
+  ck_assert_str_eq(json_string_value(json_object_get(j_resp, "verification_uri")), "http://localhost:4593/api/" PLUGIN_NAME "/device");
   ck_assert_ptr_ne(json_object_get(j_resp, "verification_uri_complete"), NULL);
   ck_assert_int_eq(json_integer_value(json_object_get(j_resp, "expires_in")), 600);
   ck_assert_int_eq(json_integer_value(json_object_get(j_resp, "interval")), 5);
@@ -765,7 +772,7 @@ START_TEST(test_oidc_device_authorization_device_verification_client_invalid)
   ck_assert_ptr_ne(json_object_get(j_resp, "user_code"), NULL);
   ck_assert_ptr_ne(code = json_string_value(json_object_get(j_resp, "user_code")), NULL);
   ck_assert_ptr_ne(device_code = json_string_value(json_object_get(j_resp, "device_code")), NULL);
-  ck_assert_str_eq(json_string_value(json_object_get(j_resp, "verification_uri")), "http://localhost:4593/api/oidc_device/device");
+  ck_assert_str_eq(json_string_value(json_object_get(j_resp, "verification_uri")), "http://localhost:4593/api/" PLUGIN_NAME "/device");
   ck_assert_ptr_ne(json_object_get(j_resp, "verification_uri_complete"), NULL);
   ck_assert_int_eq(json_integer_value(json_object_get(j_resp, "expires_in")), 600);
   ck_assert_int_eq(json_integer_value(json_object_get(j_resp, "interval")), 5);
@@ -834,7 +841,7 @@ START_TEST(test_oidc_device_authorization_device_verification_client_secret_inva
   ck_assert_ptr_ne(json_object_get(j_resp, "user_code"), NULL);
   ck_assert_ptr_ne(code = json_string_value(json_object_get(j_resp, "user_code")), NULL);
   ck_assert_ptr_ne(device_code = json_string_value(json_object_get(j_resp, "device_code")), NULL);
-  ck_assert_str_eq(json_string_value(json_object_get(j_resp, "verification_uri")), "http://localhost:4593/api/oidc_device/device");
+  ck_assert_str_eq(json_string_value(json_object_get(j_resp, "verification_uri")), "http://localhost:4593/api/" PLUGIN_NAME "/device");
   ck_assert_ptr_ne(json_object_get(j_resp, "verification_uri_complete"), NULL);
   ck_assert_int_eq(json_integer_value(json_object_get(j_resp, "expires_in")), 600);
   ck_assert_int_eq(json_integer_value(json_object_get(j_resp, "interval")), 5);
