@@ -6,6 +6,7 @@ import Notification from '../lib/Notification';
 import i18next from 'i18next';
 
 import Confirm from '../Modal/Confirm';
+import Message from '../Modal/Message';
 import EditRecord from '../Modal/EditRecord';
 
 import Navbar from './Navbar';
@@ -19,6 +20,7 @@ import Plugin from './Plugin';
 import ScopeEdit from './ScopeEdit';
 import ModEdit from './ModEdit';
 import PluginEdit from './PluginEdit';
+import APIKey from './APIKey';
 
 class App extends Component {
   constructor(props) {
@@ -36,7 +38,10 @@ class App extends Component {
       curClient: false,
       scopes: {list: [], offset: 0, limit: 20, searchPattern: "", pattern: false},
       curScope: false,
+      apiKeys: {list: [], offset: 0, limit: 20, searchPattern: "", pattern: false},
+      curApiKey: false,
       confirmModal: {title: "", message: ""},
+      messageModal: {title: "", label: "", message: []},
       editModal: {title: "", pattern: [], source: [], data: {}, callback: false, validateCallback: false, add: false},
       scopeModal: {title: "", data: {name: "", display_name: "", description: "", password_required: true, scheme: {}}, callback: false, add: false},
       curMod: false,
@@ -52,8 +57,9 @@ class App extends Component {
       savedRecord: false,
       savedIndex: -1
     }
-    
+
     this.fetchApi = this.fetchApi.bind(this);
+    this.reloadMods = this.reloadMods.bind(this);
 
     this.fetchUsers = this.fetchUsers.bind(this);
     this.confirmDeleteUser = this.confirmDeleteUser.bind(this);
@@ -72,21 +78,21 @@ class App extends Component {
     this.confirmDeleteScope = this.confirmDeleteScope.bind(this);
     this.confirmEditScope = this.confirmEditScope.bind(this);
     this.confirmAddScope = this.confirmAddScope.bind(this);
-    
+
     this.fetchModTypes = this.fetchModTypes.bind(this);
     this.fetchUserMods = this.fetchUserMods.bind(this);
     this.fetchClientMods = this.fetchClientMods.bind(this);
     this.fetchSchemeMods = this.fetchSchemeMods.bind(this);
     this.fetchPlugins = this.fetchPlugins.bind(this);
-    
+
     this.confirmAddUserMod = this.confirmAddUserMod.bind(this);
     this.confirmEditUserMod = this.confirmEditUserMod.bind(this);
     this.confirmDeleteUserMod = this.confirmDeleteUserMod.bind(this);
-    
+
     this.confirmAddClientMod = this.confirmAddClientMod.bind(this);
     this.confirmEditClientMod = this.confirmEditClientMod.bind(this);
     this.confirmDeleteClientMod = this.confirmDeleteClientMod.bind(this);
-    
+
     this.confirmAddSchemeMod = this.confirmAddSchemeMod.bind(this);
     this.confirmEditSchemeMod = this.confirmEditSchemeMod.bind(this);
     this.confirmDeleteSchemeMod = this.confirmDeleteSchemeMod.bind(this);
@@ -95,6 +101,9 @@ class App extends Component {
     this.confirmEditPluginMod = this.confirmEditPluginMod.bind(this);
     this.confirmDeletePluginMod = this.confirmDeletePluginMod.bind(this);
 
+    this.addApiKey = this.addApiKey.bind(this);
+    this.confirmDisableApiKey = this.confirmDisableApiKey.bind(this);
+
     messageDispatcher.subscribe('App', (message) => {
       if (message.type === 'nav') {
         this.setState({curNav: message.message});
@@ -106,6 +115,8 @@ class App extends Component {
         });
       } else if (message.type === 'lang') {
         this.setState({lang: i18next.language});
+      } else if (message.type === 'reloadMods') {
+        this.reloadMods();
       } else if (message.type === 'delete') {
         if (message.role === 'user') {
           var confirmModal = {
@@ -170,6 +181,15 @@ class App extends Component {
           this.setState({confirmModal: confirmModal, curMod: message.mod}, () => {
             $("#confirmModal").modal({keyboard: false, show: true});
           });
+        } else if (message.role === 'apiKey') {
+          var confirmModal = {
+            title: i18next.t("admin.confirm-delete-api-key-title"),
+            message: i18next.t("admin.confirm-delete-api-key"),
+            callback: this.confirmDisableApiKey
+          }
+          this.setState({confirmModal: confirmModal, curApiKey: message.apiKey}, () => {
+            $("#confirmModal").modal({keyboard: false, show: true});
+          });
         }
       } else if (message.type === 'edit') {
         if (message.role === 'user') {
@@ -279,7 +299,7 @@ class App extends Component {
         } else if (message.role === 'scope') {
           var scopeModal = {
             title: i18next.t("admin.add-scope-title"),
-            data: {name: "", display_name: "", description: "", password_required: true, password_max_age: 0, scheme: {}},
+            data: {name: "", display_name: "", description: "", password_required: true, password_max_age: 0, scheme: {}, schemeRequired: {}},
             callback: this.confirmAddScope,
             add: true
           }
@@ -333,6 +353,8 @@ class App extends Component {
           this.setState({PluginModal: PluginModal}, () => {
             $("#editPluginModal").modal({keyboard: false, show: true});
           });
+        } else if (message.role === 'apiKey') {
+          this.addApiKey();
         }
       } else if (message.type === 'swap') {
         if (message.role === 'userMod') {
@@ -404,12 +426,26 @@ class App extends Component {
         }
       }
     });
-    
+
     if (this.state.config) {
       this.fetchApi();
     }
   }
-  
+
+  reloadMods() {
+    apiManager.glewlwydRequest("/mod/reload", "PUT")
+    .then(() => {
+      this.fetchModTypes();
+      this.fetchUserMods();
+      this.fetchClientMods();
+      this.fetchSchemeMods();
+      this.fetchPlugins();
+      this.fetchApiKeys();
+    }).fail((err) => {
+      messageDispatcher.sendMessage('Notification', {type: "danger", message: i18next.t("admin.error-api-fetch")});
+    });
+  }
+
   fetchApi() {
     apiManager.glewlwydRequest("/profile_list")
     .then((res) => {
@@ -427,6 +463,7 @@ class App extends Component {
             this.fetchSchemeMods();
             this.fetchPlugins();
             this.fetchAllScopes();
+            this.fetchApiKeys();
           });
         })
         .fail((error) => {
@@ -435,6 +472,7 @@ class App extends Component {
             users: {list: [], offset: 0, limit: 20, searchPattern: "", pattern: false},
             clients: {list: [], offset: 0, limit: 20, searchPattern: "", pattern: false},
             scopes: {list: [], offset: 0, limit: 20, searchPattern: "", pattern: false},
+            apiKeys: {list: [], offset: 0, limit: 20, searchPattern: "", pattern: false},
             modUsers: [],
             modClients: [],
             modSchemes: [],
@@ -451,6 +489,7 @@ class App extends Component {
           users: {list: [], offset: 0, limit: 20, searchPattern: "", pattern: false},
           clients: {list: [], offset: 0, limit: 20, searchPattern: "", pattern: false},
           scopes: {list: [], offset: 0, limit: 20, searchPattern: "", pattern: false},
+          apiKeys: {list: [], offset: 0, limit: 20, searchPattern: "", pattern: false},
           modUsers: [],
           modClients: [],
           modSchemes: [],
@@ -485,6 +524,7 @@ class App extends Component {
           users: {list: [], offset: 0, limit: 20, searchPattern: "", pattern: false},
           clients: {list: [], offset: 0, limit: 20, searchPattern: "", pattern: false},
           scopes: {list: [], offset: 0, limit: 20, searchPattern: "", pattern: false},
+          apiKeys: {list: [], offset: 0, limit: 20, searchPattern: "", pattern: false},
           modUsers: [],
           modClients: [],
           modSchemes: [],
@@ -511,6 +551,7 @@ class App extends Component {
           users: {list: [], offset: 0, limit: 20, searchPattern: "", pattern: false},
           clients: {list: [], offset: 0, limit: 20, searchPattern: "", pattern: false},
           scopes: {list: [], offset: 0, limit: 20, searchPattern: "", pattern: false},
+          apiKeys: {list: [], offset: 0, limit: 20, searchPattern: "", pattern: false},
           modUsers: [],
           modClients: [],
           modSchemes: [],
@@ -537,6 +578,7 @@ class App extends Component {
           users: {list: [], offset: 0, limit: 20, searchPattern: "", pattern: false},
           clients: {list: [], offset: 0, limit: 20, searchPattern: "", pattern: false},
           scopes: {list: [], offset: 0, limit: 20, searchPattern: "", pattern: false},
+          apiKeys: {list: [], offset: 0, limit: 20, searchPattern: "", pattern: false},
           modUsers: [],
           modClients: [],
           modSchemes: [],
@@ -545,7 +587,7 @@ class App extends Component {
       }
     });
   }
-  
+
   fetchAllScopes() {
     return apiManager.glewlwydRequest("/scope?limit=0")
     .then((scopes) => {
@@ -585,6 +627,7 @@ class App extends Component {
           users: {list: [], offset: 0, limit: 20, searchPattern: "", pattern: false},
           clients: {list: [], offset: 0, limit: 20, searchPattern: "", pattern: false},
           scopes: {list: [], offset: 0, limit: 20, searchPattern: "", pattern: false},
+          apiKeys: {list: [], offset: 0, limit: 20, searchPattern: "", pattern: false},
           modUsers: [],
           modClients: [],
           modSchemes: [],
@@ -593,7 +636,7 @@ class App extends Component {
       }
     });
   }
-  
+
   fetchModTypes () {
     return apiManager.glewlwydRequest("/mod/type")
     .then((modTypes) => {
@@ -608,6 +651,7 @@ class App extends Component {
           users: {list: [], offset: 0, limit: 20, searchPattern: "", pattern: false},
           clients: {list: [], offset: 0, limit: 20, searchPattern: "", pattern: false},
           scopes: {list: [], offset: 0, limit: 20, searchPattern: "", pattern: false},
+          apiKeys: {list: [], offset: 0, limit: 20, searchPattern: "", pattern: false},
           modUsers: [],
           modClients: [],
           modSchemes: [],
@@ -616,7 +660,7 @@ class App extends Component {
       }
     });
   }
-  
+
   fetchClientMods () {
     return apiManager.glewlwydRequest("/mod/client")
     .then((modClients) => {
@@ -631,6 +675,7 @@ class App extends Component {
           users: {list: [], offset: 0, limit: 20, searchPattern: "", pattern: false},
           clients: {list: [], offset: 0, limit: 20, searchPattern: "", pattern: false},
           scopes: {list: [], offset: 0, limit: 20, searchPattern: "", pattern: false},
+          apiKeys: {list: [], offset: 0, limit: 20, searchPattern: "", pattern: false},
           modUsers: [],
           modClients: [],
           modSchemes: [],
@@ -639,7 +684,7 @@ class App extends Component {
       }
     });
   }
-  
+
   fetchSchemeMods () {
     return apiManager.glewlwydRequest("/mod/scheme")
     .then((modSchemes) => {
@@ -654,6 +699,7 @@ class App extends Component {
           users: {list: [], offset: 0, limit: 20, searchPattern: "", pattern: false},
           clients: {list: [], offset: 0, limit: 20, searchPattern: "", pattern: false},
           scopes: {list: [], offset: 0, limit: 20, searchPattern: "", pattern: false},
+          apiKeys: {list: [], offset: 0, limit: 20, searchPattern: "", pattern: false},
           modUsers: [],
           modClients: [],
           modSchemes: [],
@@ -662,7 +708,7 @@ class App extends Component {
       }
     });
   }
-  
+
   fetchPlugins () {
     return apiManager.glewlwydRequest("/mod/plugin")
     .then((plugins) => {
@@ -677,6 +723,7 @@ class App extends Component {
           users: {list: [], offset: 0, limit: 20, searchPattern: "", pattern: false},
           clients: {list: [], offset: 0, limit: 20, searchPattern: "", pattern: false},
           scopes: {list: [], offset: 0, limit: 20, searchPattern: "", pattern: false},
+          apiKeys: {list: [], offset: 0, limit: 20, searchPattern: "", pattern: false},
           modUsers: [],
           modClients: [],
           modSchemes: [],
@@ -685,7 +732,35 @@ class App extends Component {
       }
     });
   }
-  
+
+  fetchApiKeys () {
+    return apiManager.glewlwydRequest("/key?offset=" + this.state.apiKeys.offset + "&limit=" + this.state.apiKeys.limit + (this.state.apiKeys.searchPattern?"&pattern="+this.state.apiKeys.searchPattern:""))
+    .then((apiKeys) => {
+      var curApiKeys = this.state.apiKeys;
+      curApiKeys.list = apiKeys;
+      curApiKeys.pattern = this.state.config.pattern.user;
+      this.setState({apiKeys: curApiKeys, loggedIn: true});
+    }).fail((err) => {
+      if (err.status !== 401) {
+        messageDispatcher.sendMessage('Notification', {type: "danger", message: i18next.t("admin.error-api-fetch")});
+      } else {
+        messageDispatcher.sendMessage('Notification', {type: "danger", message: i18next.t("admin.requires-admin-scope")});
+        this.setState({
+          loggedIn: false,
+          modTypes: {user: [], client: [], scheme: [], plugin: []},
+          users: {list: [], offset: 0, limit: 20, searchPattern: "", pattern: false},
+          clients: {list: [], offset: 0, limit: 20, searchPattern: "", pattern: false},
+          scopes: {list: [], offset: 0, limit: 20, searchPattern: "", pattern: false},
+          apiKeys: {list: [], offset: 0, limit: 20, searchPattern: "", pattern: false},
+          modUsers: [],
+          modClients: [],
+          modSchemes: [],
+          plugins: []
+        });
+      }
+    });
+  }
+
   confirmDeleteUser(result) {
     if (result) {
       apiManager.glewlwydRequest("/user/" + encodeURIComponent(this.state.curUser.username), "DELETE")
@@ -911,7 +986,7 @@ class App extends Component {
   validateUser(user, confirmData, add, cb) {
     var result = true, data = {};
     if (add) {
-      if (user.password !== undefined || confirmData.password !== undefined) {
+      if (!Array.isArray(user.password) && (user.password !== undefined || confirmData.password !== undefined)) {
         if (user.password !== confirmData.password) {
           result = false;
           data["password"] = i18next.t("admin.user-password-error-match");
@@ -935,7 +1010,7 @@ class App extends Component {
         });
       }
     } else {
-      if (user.password !== undefined || confirmData.password !== undefined) {
+      if (!Array.isArray(user.password) && (user.password !== undefined || confirmData.password !== undefined)) {
         if (user.password || confirmData.password) {
           if (user.password !== confirmData.password) {
             result = false;
@@ -987,7 +1062,7 @@ class App extends Component {
       cb(result, data);
     }
   }
-  
+
   confirmAddUserMod(result, mod) {
     if (result) {
       apiManager.glewlwydRequest("/mod/user/", "POST", mod)
@@ -1330,6 +1405,37 @@ class App extends Component {
     }
   }
 
+  addApiKey() {
+    apiManager.glewlwydRequest("/key", "POST")
+    .then((result) => {
+      var messageModal = this.state.messageModal;
+      messageModal.title = i18next.t("admin.api-key-add-title");
+      messageModal.label = i18next.t("admin.api-key-add-label");
+      messageModal.message = [result.key];
+      this.setState({messageModal: messageModal}, () => {
+        $("#messageModal").modal({keyboard: false, show: true});
+      });
+      this.fetchApiKeys();
+    })
+    .fail(() => {
+      messageDispatcher.sendMessage('Notification', {type: "danger", message: i18next.t("error-api-connect")});
+    })
+  }
+
+  confirmDisableApiKey() {
+    apiManager.glewlwydRequest("/key/" + encodeURIComponent(this.state.curApiKey.token_hash), "DELETE")
+    .then((key) => {
+      messageDispatcher.sendMessage('Notification', {type: "success", message: i18next.t("admin.success-api-delete-api-key")});
+    })
+    .fail(() => {
+      messageDispatcher.sendMessage('Notification', {type: "danger", message: i18next.t("error-api-connect")});
+    })
+    .always(() => {
+      $("#confirmModal").modal("hide");
+      this.fetchApiKeys();
+    });
+  }
+
 	render() {
     var invalidCredentialMessage;
     if (this.state.invalidCredentialMessage) {
@@ -1367,16 +1473,46 @@ class App extends Component {
                   <div className={"carousel-item" + (this.state.curNav==="plugins"?" active":"")}>
                     <Plugin mods={this.state.plugins} types={this.state.modTypes.plugin} loggedIn={this.state.loggedIn} />
                   </div>
+                  <div className={"carousel-item" + (this.state.curNav==="api-key"?" active":"")}>
+                    <APIKey config={this.state.config} apiKeys={this.state.apiKeys} loggedIn={this.state.loggedIn} />
+                  </div>
                 </div>
               </div>
             </div>
           </div>
           <Notification loggedIn={this.state.loggedIn}/>
-          <Confirm title={this.state.confirmModal.title} message={this.state.confirmModal.message} callback={this.state.confirmModal.callback} />
-          <EditRecord title={this.state.editModal.title} pattern={this.state.editModal.pattern} source={this.state.editModal.source} data={this.state.editModal.data} callback={this.state.editModal.callback} validateCallback={this.state.editModal.validateCallback} add={this.state.editModal.add} />
-          <ScopeEdit title={this.state.scopeModal.title} scope={this.state.scopeModal.data} add={this.state.scopeModal.add} modSchemes={this.state.modSchemes} callback={this.state.scopeModal.callback} />
-          <ModEdit title={this.state.ModModal.title} role={this.state.ModModal.role} mod={this.state.ModModal.data} add={this.state.ModModal.add} types={this.state.ModModal.types} callback={this.state.ModModal.callback} config={this.state.config} />
-          <PluginEdit title={this.state.PluginModal.title} mod={this.state.PluginModal.data} add={this.state.PluginModal.add} modSchemes={this.state.modSchemes} types={this.state.PluginModal.types} callback={this.state.PluginModal.callback} config={this.state.config} />
+          <Confirm title={this.state.confirmModal.title}
+                   message={this.state.confirmModal.message}
+                   callback={this.state.confirmModal.callback} />
+          <Message title={this.state.messageModal.title}
+                   label={this.state.messageModal.label}
+                   message={this.state.messageModal.message} />
+          <EditRecord title={this.state.editModal.title}
+                      pattern={this.state.editModal.pattern}
+                      source={this.state.editModal.source}
+                      data={this.state.editModal.data}
+                      callback={this.state.editModal.callback}
+                      validateCallback={this.state.editModal.validateCallback}
+                      add={this.state.editModal.add} />
+          <ScopeEdit title={this.state.scopeModal.title}
+                     scope={this.state.scopeModal.data}
+                     add={this.state.scopeModal.add}
+                     modSchemes={this.state.modSchemes}
+                     callback={this.state.scopeModal.callback} />
+          <ModEdit title={this.state.ModModal.title}
+                   role={this.state.ModModal.role}
+                   mod={this.state.ModModal.data}
+                   add={this.state.ModModal.add}
+                   types={this.state.ModModal.types}
+                   callback={this.state.ModModal.callback}
+                   config={this.state.config} />
+          <PluginEdit title={this.state.PluginModal.title}
+                      mod={this.state.PluginModal.data}
+                      add={this.state.PluginModal.add}
+                      modSchemes={this.state.modSchemes}
+                      types={this.state.PluginModal.types}
+                      callback={this.state.PluginModal.callback}
+                      config={this.state.config} />
         </div>
       );
     } else {
