@@ -12,6 +12,9 @@ import SelectAccount from './SelectAccount';
 import EndSession from './EndSession';
 import SessionClosed from './SessionClosed';
 import DeviceAuth from './DeviceAuth';
+import ResetCredentials from './ResetCredentials';
+
+import Message from '../Modal/Message';
 
 class App extends Component {
   constructor(props) {
@@ -26,11 +29,13 @@ class App extends Component {
       scope: [],
       mustRegisterScheme: false,
       scheme: props.config.params.scheme,
+      identify: this.isSchemeIdentify(props.config.params.scheme, props.config.sessionSchemes),
       schemeListRequired: false,
       passwordRequired: false,
       client: false,
       showGrant: true,
       showGrantAsterisk: false,
+      showAuthDetailsAsterisk: false,
       canContinue: false,
       prompt: props.config.params.prompt,
       refresh_login: props.config.params.refresh_login,
@@ -42,7 +47,16 @@ class App extends Component {
       login_hint: props.config.params.login_hint||"",
       errorScopesUnavailable: false,
       infoSomeScopeUnavailable: false,
-      errorScheme: false
+      errorScheme: false,
+      registration: [],
+      resetCredentials: [],
+      resetCredentialsShow: false,
+      authDetails: [],
+      messageModal: {
+        title: "",
+        label: "",
+        message: ""
+      }
     };
 
     this.initProfile = this.initProfile.bind(this);
@@ -59,14 +73,13 @@ class App extends Component {
       if (message.type === "InitProfile") {
         this.initProfile(false);
       } else if (message.type === "loginSuccess") {
-        this.setState({selectAccount: false, newUser: false, refresh_login: false, prompt: false}, () => {
+        this.setState({selectAccount: false, newUser: false, refresh_login: false, prompt: false, resetCredentialsShow: false}, () => {
           this.initProfile(false);
         });
       } else if (message.type === "NewUser") {
-        this.setState({selectAccount: false, newUser: true, currentUser: false, scheme: this.state.config.params.scheme}, () => {
-        });
+        this.setState({selectAccount: false, newUser: true, currentUser: false, scheme: this.state.config.params.scheme, identify: false, resetCredentialsShow: false});
       } else if (message.type === "GrantComplete") {
-        this.setState({selectAccount: false, showGrant: false, prompt: false, forceShowGrant: false}, () => {
+        this.setState({selectAccount: false, showGrant: false, prompt: false, forceShowGrant: false, resetCredentialsShow: false}, () => {
           this.initProfile(false);
         });
       } else if (message.type === "SelectAccount") {
@@ -80,14 +93,70 @@ class App extends Component {
       } else if (message.type === "ToggleGrant") {
         this.setState({showGrant: !this.state.showGrant});
       } else if (message.type === "newUserScheme") {
-        this.setState({scheme: message.scheme});
+        this.setState({scheme: message.scheme, identify: message.identify});
       } else if (message.type === "SessionClosed") {
         this.setState({endSession: false, sessionClosed: true});
+      } else if (message.type === "SessionReopen") {
+        this.setState({sessionClosed: false, newUser: true, currentUser: false, login_hint: ""}, () => {
+          this.initProfile(true);
+        });
+      } else if (message.type === "ResetCredentials") {
+        this.setState({selectAccount: false, newUser: false, refresh_login: false, prompt: false, resetCredentialsShow: true});
+      } else if (message.type === 'message') {
+        var messageModal = this.state.messageModal;
+        messageModal.title = message.title;
+        messageModal.label = message.label;
+        messageModal.message = message.message;
+        this.setState({messageModal: messageModal}, () => {
+          $("#messageModal").modal({keyboard: false, show: true});
+        });
       }
     });
   }
+  
+  isSchemeIdentify(scheme, sessionSchemes) {
+    if (sessionSchemes) {
+      var identify = false;
+      sessionSchemes.forEach(curScheme => {
+        if (curScheme.scheme_name === scheme) {
+          identify = curScheme.identify;
+        }
+      });
+      return identify;
+    } else {
+      return false;
+    }
+  }
 
   initProfile(checkPrompt) {
+    if (this.state.config.register) {
+      var registration = [];
+      var resetCredentials = [];
+      this.state.config.register.forEach((register, index) => {
+        apiManager.glewlwydRequest("/" + register.name + "/config")
+        .then((config) => {
+          if (config.registration) {
+            registration.push({name: register.name, message: register.message});
+          }
+          if (config["reset-credentials"].email || config["reset-credentials"].code) {
+            resetCredentials.push({name: register.name,
+                                   message: register["reset-credentials-message"],
+                                   email: config["reset-credentials"].email,
+                                   code: config["reset-credentials"].code});
+          }
+          this.setState({registration: registration, resetCredentials: resetCredentials});
+        })
+        .fail((err) => {
+          this.setState({registerValid: false, registering: false}, () => {
+            if (err.status === 404) {
+              messageDispatcher.sendMessage('Notification', {type: "danger", message: i18next.t("profile.register-invalid-url")});
+            } else {
+              messageDispatcher.sendMessage('Notification', {type: "danger", message: i18next.t("error-api-connect")});
+            }
+          });
+        });
+      });
+    }
     apiManager.glewlwydRequest("/profile_list")
     .then((res) => {
       var newState = {};
@@ -118,7 +187,10 @@ class App extends Component {
       }
       this.setState(newState, () => {
         if (this.state.config.params.client_id && this.state.config.params.scope) {
-          this.checkClientScope(this.state.config.params.client_id, this.state.config.params.scope);
+          this.checkClientScope(this.state.config.params.client_id, this.state.config.params.scope)
+          .then(() => {
+            this.checkAuthorizationDetails(this.state.config.params.authorization_details);
+          });
         } else if (this.state.config.params.scope) {
           this.checkScopeScheme(this.state.config.params.scope);
         } else {
@@ -131,19 +203,24 @@ class App extends Component {
         messageDispatcher.sendMessage('Notification', {type: "warning", message: i18next.t("error-api-connect")});
       }
       if (this.state.prompt === "device") {
-        this.setState({deviceAuth: true, currentUser: false, userList: [], loaded: true});
+        this.setState({deviceAuth: true, currentUser: false, userList: [], loaded: true, scheme: this.state.config.params.scheme});
       } else {
-        this.setState({newUser: (!!this.state.config.params.callback_url && !!this.state.config.params.scope), showGrant: false, currentUser: false, userList: [], loaded: true});
+        this.setState({newUser: (!!this.state.config.params.callback_url && !!this.state.config.params.scope), showGrant: false, currentUser: false, userList: [], loaded: true, scheme: this.state.config.params.scheme});
       }
     });
   }
 
   checkClientScope(clientId, scopeList) {
-    apiManager.glewlwydRequest("/auth/grant/" + encodeURIComponent(clientId) + "/" + encodeURIComponent(scopeList))
+    return apiManager.glewlwydRequest("/auth/grant/" + encodeURIComponent(clientId) + "/" + encodeURIComponent(scopeList))
     .then((res) => {
       var scopeGranted = [];
       var showGrant = true;
       var showGrantAsterisk = false;
+      var callback_url = decodeURIComponent(this.state.config.params.callback_url);
+      if (callback_url) {
+        const urlParams = new URLSearchParams(callback_url);
+        res.client.redirect_uri = urlParams.get("redirect_uri");
+      }
       if (res.scope.length) {
         var infoSomeScopeUnavailable = (scopeList.split(" ").length > res.scope.length);
         if (scopeList === "openid") {
@@ -165,31 +242,31 @@ class App extends Component {
           });
         }
         if (scopeGranted.length) {
-          apiManager.glewlwydRequest("/auth/scheme/?scope=" + encodeURIComponent(scopeGranted.join(" ")))
+          return apiManager.glewlwydRequest("/auth/scheme/?scope=" + encodeURIComponent(scopeGranted.join(" ")))
           .then((schemeRes) => {
-            this.setState({client: res.client, 
-                           scope: res.scope, 
-                           scheme: schemeRes, 
-                           showGrant: showGrant, 
-                           showGrantAsterisk: showGrantAsterisk, 
-                           infoSomeScopeUnavailable: infoSomeScopeUnavailable, 
+            return this.setState({client: res.client,
+                           scope: res.scope,
+                           scheme: schemeRes,
+                           showGrant: showGrant,
+                           showGrantAsterisk: showGrantAsterisk,
+                           infoSomeScopeUnavailable: infoSomeScopeUnavailable,
                            errorScopesUnavailable: false}, () => {
-              this.parseSchemes();
+              return this.parseSchemes();
             });
           })
           .fail((error) => {
             messageDispatcher.sendMessage('Notification', {type: "warning", message: i18next.t("login.error-scheme-scope-api")});
           });
         } else {
-          this.setState({client: res.client, 
-                         scope: res.scope, 
-                         showGrant: true, 
-                         showGrantAsterisk: true, 
-                         errorScopesUnavailable: false, 
+          return this.setState({client: res.client,
+                         scope: res.scope,
+                         showGrant: true,
+                         showGrantAsterisk: true,
+                         errorScopesUnavailable: false,
                          infoSomeScopeUnavailable: infoSomeScopeUnavailable});
         }
       } else {
-        this.setState({errorScopesUnavailable: true, infoSomeScopeUnavailable: false});
+        return this.setState({errorScopesUnavailable: true, infoSomeScopeUnavailable: false});
       }
     })
     .fail((error) => {
@@ -217,6 +294,36 @@ class App extends Component {
     });
   }
 
+  checkAuthorizationDetails(authorization_details) {
+    if (authorization_details) {
+      var showAuthDetailsAsterisk = false;
+      var authDetails = [];
+      authorization_details.split(",").forEach(authName => {
+        apiManager.glewlwydRequest("/" + this.state.config.params.plugin + "/rar/" + this.state.config.params.client_id + "/" +authName)
+        .then((curDetails) => {
+          if (curDetails.scopes && curDetails.scopes.length) {
+            var enabled = false;
+            this.state.scope.forEach((scope) => {
+              if (curDetails.scopes.indexOf(scope.name) > -1) {
+                enabled = true;
+              }
+            });
+            curDetails.enabled = enabled;
+          } else {
+            curDetails.enabled = true;
+          }
+          authDetails.push(curDetails);
+          this.setState({authDetails: authDetails, showAuthDetailsAsterisk: (showAuthDetailsAsterisk | !curDetails.consent)});
+        })
+        .fail((error) => {
+          if (error.status !== 404) {
+            messageDispatcher.sendMessage('Notification', {type: "warning", message: i18next.t("login.error-scheme-scope-api")});
+          }
+        });
+      });
+    }
+  }
+
   parseSchemes() {
     var canContinue = !!this.state.config.params.callback_url;
     var passwordRequired = false;
@@ -235,23 +342,24 @@ class App extends Component {
         } else if (!schemeListRequired && canContinue) {
           for (var groupName in scope.schemes) {
             var group = scope.schemes[groupName];
-            var groupAuthenticated = false;
+            var schemeRequired = scope.scheme_required[groupName];
+            var groupSchemeAuthenticated = 0;
             schemeListRequired = group;
             group.forEach((curScheme) => {
               mustRegisterScheme = false;
               if (curScheme.scheme_authenticated) {
-                groupAuthenticated = true;
-                schemeListRequired = false;
-                scheme = false;
+                groupSchemeAuthenticated++;
               } else if ((!scheme || scheme.scheme_last_login < curScheme.scheme_last_login) && curScheme.scheme_registered) {
                 scheme = curScheme;
               } else if (!curScheme.scheme_registered && !scheme) {
                 mustRegisterScheme = true;
               }
             });
-            if (!groupAuthenticated) {
+            if (groupSchemeAuthenticated < schemeRequired) {
               canContinue = false;
               break;
+            } else {
+              schemeListRequired = false;
             }
           }
         }
@@ -263,11 +371,11 @@ class App extends Component {
     if (canContinue) {
       scheme = false;
     }
-    this.setState({canContinue: canContinue, 
-                   passwordRequired: passwordRequired, 
-                   schemeListRequired: schemeListRequired, 
-                   scheme: scheme, 
-                   errorScheme: (!scheme && !canContinue), 
+    return this.setState({canContinue: canContinue,
+                   passwordRequired: passwordRequired,
+                   schemeListRequired: schemeListRequired,
+                   scheme: scheme,
+                   errorScheme: (!scheme && !canContinue),
                    mustRegisterScheme: mustRegisterScheme});
   }
 
@@ -277,7 +385,7 @@ class App extends Component {
       this.setState({lang: lang});
     });
   }
-  
+
   userHasScope(user, scope_list) {
     var hasScope = false;
     if (scope_list) {
@@ -294,7 +402,9 @@ class App extends Component {
     if (this.state.config) {
       var body = "", message, scopeUnavailable;
       if (this.state.loaded) {
-        if (this.state.endSession) {
+        if (this.state.resetCredentialsShow) {
+          body = <ResetCredentials config={this.state.config} resetCredentials={this.state.resetCredentials}/>;
+        } else if (this.state.endSession) {
           body = <EndSession config={this.state.config} userList={this.state.userList} currentUser={this.state.currentUser}/>;
         } else if (this.state.sessionClosed) {
           body = <SessionClosed config={this.state.config}/>;
@@ -317,22 +427,32 @@ class App extends Component {
           }
           if ((this.state.newUser || this.state.passwordRequired)) {
             if (!this.state.scheme) {
-              body = <PasswordForm config={this.state.config} username={this.state.login_hint} currentUser={this.state.currentUser} userList={this.state.userList} callbackInitProfile={this.initProfile}/>;
+              body = <PasswordForm config={this.state.config}
+                                   username={this.state.login_hint}
+                                   currentUser={this.state.currentUser}
+                                   userList={this.state.userList}
+                                   callbackInitProfile={this.initProfile}/>;
             } else {
-              body = <NoPasswordForm config={this.state.config} username={this.state.login_hint} userList={this.state.userList} callbackInitProfile={this.initProfile} scheme={this.state.scheme}/>;
+              body = <NoPasswordForm config={this.state.config}
+                                     username={this.state.login_hint}
+                                     userList={this.state.userList}
+                                     callbackInitProfile={this.initProfile}
+                                     identify={this.state.identify}
+                                     scheme={this.state.scheme}/>;
             }
           } else if (this.state.selectAccount) {
             body = <SelectAccount config={this.state.config} userList={this.state.userList} currentUser={this.state.currentUser}/>;
           } else {
-            body = <Body config={this.state.config} 
-                         currentUser={this.state.currentUser} 
-                         client={this.state.client} 
-                         scope={this.state.scope} 
-                         scheme={this.state.scheme} 
-                         schemeListRequired={this.state.schemeListRequired} 
-                         showGrant={this.state.showGrant} 
+            body = <Body config={this.state.config}
+                         currentUser={this.state.currentUser}
+                         client={this.state.client}
+                         scope={this.state.scope}
+                         scheme={this.state.scheme}
+                         schemeListRequired={this.state.schemeListRequired}
+                         showGrant={this.state.showGrant}
                          infoSomeScopeUnavailable={this.state.infoSomeScopeUnavailable}
-                         validLogin={(!!this.state.config.params.callback_url && !!this.state.config.params.scope)}/>;
+                         validLogin={(!!this.state.config.params.callback_url && !!this.state.config.params.scope)}
+                         authDetails={this.state.authDetails}/>;
             if (this.state.errorScopesUnavailable) {
               scopeUnavailable = <div className="alert alert-danger" role="alert">{i18next.t("login.error-scope-unavailable")}</div>
             }
@@ -360,9 +480,6 @@ class App extends Component {
                   <span className="navbar-toggler-icon"></span>
                 </button>
                 <div className="collapse navbar-collapse" id="navbarSupportedContent">
-                  <ul className="navbar-nav mr-auto">
-                    {/* Placeholder */}
-                  </ul>
                   <div className="btn-group" role="group">
                     <button className="btn btn-secondary dropdown-toggle" type="button" id="dropdownLang" data-toggle="dropdown" aria-haspopup="true" aria-expanded="false">
                       <i className="fas fa-language"></i>
@@ -380,20 +497,25 @@ class App extends Component {
               {body}
             </div>
             <div className="card-footer">
-              <Buttons config={this.state.config} 
-                       currentUser={this.state.currentUser} 
+              <Buttons config={this.state.config}
+                       currentUser={this.state.currentUser}
                        userList={this.state.userList}
                        client={this.state.client}
-                       showGrant={this.state.showGrant} 
-                       showGrantAsterisk={this.state.showGrantAsterisk} 
-                       newUser={this.state.newUser} 
-                       newUserScheme={this.state.scheme} 
-                       canContinue={this.state.canContinue && !this.state.errorScopesUnavailable} 
+                       showGrant={this.state.showGrant}
+                       showGrantAsterisk={this.state.showGrantAsterisk || this.state.showAuthDetailsAsterisk}
+                       newUser={this.state.newUser}
+                       newUserScheme={this.state.scheme}
+                       canContinue={this.state.canContinue && !this.state.errorScopesUnavailable}
                        schemeListRequired={this.state.schemeListRequired}
-                       selectAccount={this.state.selectAccount} />
+                       selectAccount={this.state.selectAccount}
+                       registration={this.state.registration}
+                       resetCredentials={this.state.resetCredentials}
+                       resetCredentialsShow={this.state.resetCredentialsShow} 
+                       sessionClosed={this.state.sessionClosed}/>
             </div>
           </div>
           <Notification loggedIn={true}/>
+          <Message title={this.state.messageModal.title} label={this.state.messageModal.label} message={this.state.messageModal.message} />
         </div>
       );
     } else {
